@@ -18,38 +18,37 @@
 //
 //-----------------------------------------------------------------------------
 
-#ifndef SEQ24_CONFIGFILE
-#define SEQ24_CONFIGFILE
+#pragma once
 
 #include "perform.h"
 #include <fstream>
 #include <string>
 #include <list>
 
-class configfile 
+class configfile
 {
 
  protected:
-    
+
     int m_pos;
-    string m_name;
+    Glib::ustring m_name;
 
     /* holds our data */
     unsigned char *m_d;
-    
+
     list<unsigned char> m_l;
 
     char m_line[1024];
 
     bool m_done;
-    
+
     void next_data_line( ifstream *a_file);
     void line_after( ifstream *a_file, string a_tag);
 
  public:
 
-    configfile( string a_name );
-    virtual ~configfile(  );
+    configfile(const Glib::ustring& a_name);
+    virtual ~configfile();
 
     virtual bool parse( perform *a_perf ) = 0;
     virtual bool write( perform *a_perf ) = 0;
@@ -57,4 +56,3 @@ class configfile
 };
 
 
-#endif 
