@@ -20,49 +20,38 @@
 #include "event.h"
 #include "string.h"
 
-event::event()
+event::event() :
+    m_timestamp(0),
+    m_status(EVENT_NOTE_OFF),
+    m_linked(NULL),
+    m_has_link(false),
+    m_selected(false),
+    m_marked(false),
+    m_painted(false)
 {
-    m_timestamp = 0;
-    m_status = EVENT_NOTE_OFF;
     m_data[0] = 0;
     m_data[1] = 0;
-
-    m_sysex = NULL;
-
-    m_linked = NULL;
-    m_selected = false;
-    m_marked = false;
-    m_has_link = false;
-    m_painted = false;
-} 
-
-event::~event()
-{
-  if ( m_sysex != NULL )
-    delete[] m_sysex;
-  
-  m_sysex = NULL;
 }
 
-long 
+long
 event::get_timestamp()
-{ 
-    return m_timestamp; 
+{
+    return m_timestamp;
 }
 
-void 
+void
 event::set_timestamp( const unsigned long a_time )
 {
     m_timestamp = a_time;
 }
 
-void 
+void
 event::mod_timestamp( unsigned long a_mod )
 {
     m_timestamp %= a_mod;
 }
 
-void 
+void
 event::set_status( const char a_status  )
 {
    /* bitwise AND to clear the channel portion of the status */
@@ -72,74 +61,70 @@ event::set_status( const char a_status  )
       m_status = (char) (a_status & EVENT_CLEAR_CHAN_MASK);
 }
 
-void 
+void
 event::make_clock( )
 {
     m_status = (unsigned char) EVENT_MIDI_CLOCK;
 }
 
-void 
+void
 event::set_data( char a_D1  )
 {
     m_data[0] = a_D1 & 0x7F;
 }
 
-void 
+void
 event::set_data( char a_D1, char a_D2 )
 {
     m_data[0] = a_D1 & 0x7F;
     m_data[1] = a_D2 & 0x7F;
 }
 
-void 
-event::increment_data2(void )
+void
+event::increment_data2()
 {
 	m_data[1] = (m_data[1]+1) & 0x7F;
 }
 
-void 
-event::decrement_data2(void )
+void
+event::decrement_data2()
 {
 	m_data[1] = (m_data[1]-1) & 0x7F;
 }
 
 
 
-void 
-event::increment_data1(void )
+void
+event::increment_data1()
 {
 	m_data[0] = (m_data[0]+1) & 0x7F;
 }
 
-void 
-event::decrement_data1(void )
+void
+event::decrement_data1()
 {
 	m_data[0] = (m_data[0]-1) & 0x7F;
 }
 
 
-void 
+void
 event::get_data( unsigned char *D0, unsigned char *D1 )
 {
-    *D0 = m_data[0]; 
+    *D0 = m_data[0];
     *D1 = m_data[1];
 }
 
-unsigned char 
-event::get_status( ) 
-{ 
-    return m_status; 
+unsigned char
+event::get_status( )
+{
+    return m_status;
 }
 
 
-void 
-event::start_sysex( void  )
+void
+event::start_sysex()
 {
-  if ( m_sysex != NULL )
-    delete[] m_sysex;
-
-  m_sysex = NULL;
-  m_size = 0;
+  m_sysex.clear();
 }
 
 bool
@@ -147,33 +132,22 @@ event::append_sysex( unsigned char *a_data, long a_size )
 {
   bool ret = true;
 
-  unsigned char *buffer = new unsigned char[m_size + a_size];
-
-  /* copy old and append */
-  memcpy(  buffer        , m_sysex, m_size );
-  memcpy( &buffer[m_size], a_data, a_size );
-
-  if ( m_sysex != NULL )
-    delete[] m_sysex;
-
-  m_size = m_size + a_size;
-  m_sysex = buffer;
-
   for ( int i=0; i<a_size; i++ ){
 
+    m_sysex.push_back( a_data[i] );
     if ( a_data[i] == EVENT_SYSEX_END )
       ret = false;
   }
-    
+
   return ret;
 
 }
 
 
 unsigned char *
-event::get_sysex( void )
+event::get_sysex()
 {
-  return m_sysex;
+  return m_sysex.data();
 }
 
 
@@ -181,63 +155,63 @@ event::get_sysex( void )
 void
 event::set_size( long a_size )
 {
-  m_size = a_size;
+  m_sysex.resize(a_size);
 }
 
 long
-event::get_size( void )
+event::get_size()
 {
-  return m_size;
+  return m_sysex.size();
 }
 
-void 
+void
 event::set_note_velocity( int a_vel )
 {
-    m_data[1] = a_vel & 0x7F; 
+    m_data[1] = a_vel & 0x7F;
 }
 
-bool 
+bool
 event::is_note_on()
 {
-    return (m_status == EVENT_NOTE_ON); 
+    return (m_status == EVENT_NOTE_ON);
 }
 
-bool 
+bool
 event::is_note_off()
 {
     return (m_status == EVENT_NOTE_OFF);
 }
 
-unsigned char 
+unsigned char
 event::get_note()
 {
     return m_data[0];
 }
 
-void 
+void
 event::set_note( char a_note )
 {
     m_data[0] = a_note & 0x7F;
 }
 
-unsigned char 
+unsigned char
 event::get_note_velocity()
 {
     return m_data[1];
 }
 
 
-void 
+void
 event::print()
 {
     printf( "[%06ld] [%04lX] %02X ",
 	    m_timestamp,
-	    m_size,
+	    m_sysex.size(),
 	    m_status );
 
     if ( m_status == EVENT_SYSEX ){
 
-      for( int i=0; i<m_size; i++ ){
+      for( size_t i=0; i<m_sysex.size(); i++ ){
 
 	if ( i%16 == 0 )
 	  printf( "\n    " );
@@ -256,8 +230,8 @@ event::print()
     }
 }
 
-int 
-event::get_rank( void ) const
+int
+event::get_rank(void) const
 {
     switch ( m_status )
     {
@@ -268,9 +242,9 @@ event::get_rank( void ) const
 
         case EVENT_AFTERTOUCH:
         case EVENT_CHANNEL_PRESSURE:
-        case EVENT_PITCH_WHEEL: 
+        case EVENT_PITCH_WHEEL:
             return 0x050;
-            
+
         case EVENT_CONTROL_CHANGE:
             return 0x010;
         case EVENT_PROGRAM_CHANGE:
@@ -280,7 +254,7 @@ event::get_rank( void ) const
     }
 }
 
-bool 
+bool
 event::operator>( const event &a_rhsevent )
 {
     if ( m_timestamp == a_rhsevent.m_timestamp )
@@ -294,9 +268,9 @@ event::operator>( const event &a_rhsevent )
 }
 
 
-bool 
+bool
 event::operator<( const event &a_rhsevent )
-{ 
+{
     if ( m_timestamp == a_rhsevent.m_timestamp )
     {
         return (get_rank() < a_rhsevent.get_rank());
@@ -307,21 +281,21 @@ event::operator<( const event &a_rhsevent )
     }
 }
 
-bool 
+bool
 event::operator<=( const unsigned long &a_rhslong )
-{ 
-    return (m_timestamp <= a_rhslong); 
-}   
+{
+    return (m_timestamp <= a_rhslong);
+}
 
 
 
-bool 
+bool
 event::operator>( const unsigned long &a_rhslong )
-{ 
-    return (m_timestamp > a_rhslong); 
-}   
+{
+    return (m_timestamp > a_rhslong);
+}
 
-void 
+void
 event::link( event *a_event )
 {
     m_has_link = true;
@@ -346,54 +320,54 @@ event::clear_link( )
     m_has_link = false;
 }
 
-void 
+void
 event::select( )
 {
     m_selected = true;
 }
 
-void 
+void
 event::unselect( )
 {
     m_selected = false;
 }
 
-bool 
+bool
 event::is_selected( )
 {
     return m_selected;
 }
-void 
+void
 event::paint( )
 {
     m_painted = true;
 }
 
-void 
+void
 event::unpaint( )
 {
     m_painted = false;
 }
 
-bool 
+bool
 event::is_painted( )
 {
     return m_painted;
 }
-    
-void 
+
+void
 event::mark( )
 {
     m_marked = true;
 }
 
-void 
+void
 event::unmark( )
 {
     m_marked = false;
 }
 
-bool 
+bool
 event::is_marked( )
 {
     return m_marked;
