@@ -19,8 +19,7 @@
 //-----------------------------------------------------------------------------
 
 
-#ifndef SEQ24_SEQTIME
-#define SEQ24_SEQTIME
+#pragma once
 
 #include "globals.h"
 
@@ -32,7 +31,7 @@ const int c_pill_width = 8;
 class maintime: public Gtk::DrawingArea
 {
 
- private: 
+ private:
 
     Glib::RefPtr<Gdk::GC>       m_gc;
     Glib::RefPtr<Gdk::Window>   m_window;
@@ -50,4 +49,3 @@ class maintime: public Gtk::DrawingArea
 
 };
 
-#endif
