@@ -17,41 +17,52 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 //-----------------------------------------------------------------------------
+
 #include "midibus.h"
-#include "config.h"
-#include "sys/poll.h"
-#include "lash.h"
 
+#ifdef HAVE_LIBASOUND
+#    include <sys/poll.h>
+#endif
+
+#ifdef LASH_SUPPORT
+#    include "lash.h"
+#endif
+
+
+#ifdef HAVE_LIBASOUND
 midibus::midibus( int a_localclient,
-		  int a_destclient, 
-		  int a_destport, 
-		  snd_seq_t *a_seq, 
-		  const char *a_client_name, 
+		  int a_destclient,
+		  int a_destport,
+		  snd_seq_t *a_seq,
+		  const char *a_client_name,
 		  const char *a_port_name,
-		  int a_id, int a_queue )
+		  int a_id, int a_queue ) :
+    m_id(a_id),
+
+    m_clock_type(e_clock_off),
+    m_inputing(false),
+
+    m_seq(a_seq),
+
+    m_dest_addr_client(a_destclient),
+    m_dest_addr_port(a_destport),
+
+    m_local_addr_client(a_localclient),
+    m_local_addr_port(-1),
+
+    m_queue(a_queue)
 {
-    /* set members */
-    m_local_addr_client = a_localclient;
-    m_dest_addr_client = a_destclient;
-    m_dest_addr_port   = a_destport;
-    m_seq            = a_seq;
-    m_queue          = a_queue;
-
-    m_id = a_id;
-    m_clock_type = e_clock_off;
-    m_inputing = false;
-
-
     char name[60];
     if ( global_user_midi_bus_definitions[m_id].alias.length() > 0 )
     {
-        snprintf(name,59,"(%s)",global_user_midi_bus_definitions[m_id].alias.c_str());
+        snprintf(name, 59, "(%s)",
+                global_user_midi_bus_definitions[m_id].alias.c_str());
     }
     else
     {
         snprintf(name,59,"(%s)",a_port_name);
     }
-    
+
     /* copy names */
     char tmp[60];
     snprintf( tmp, 59, "[%d] %d:%d %s",
@@ -64,14 +75,34 @@ midibus::midibus( int a_localclient,
 }
 
 midibus::midibus( int a_localclient,
-		  snd_seq_t *a_seq, 
-		  int a_id, int a_queue )
+		  snd_seq_t *a_seq,
+		  int a_id, int a_queue ) :
+    m_id(a_id),
+    m_clock_type(e_clock_off),
+    m_inputing(false),
+    m_seq(a_seq),
+    m_dest_addr_client(-1),
+    m_dest_addr_port(-1),
+    m_local_addr_client(a_localclient),
+    m_queue(a_queue)
 {
     /* set members */
-    m_local_addr_client = a_localclient;
-    m_seq            = a_seq;
-    m_queue          = a_queue;
 
+    /* copy names */
+    char tmp[60];
+    snprintf( tmp, 59, "[%d] seq24 %d",
+	      m_id,
+	      m_id );
+
+    m_name = tmp;
+}
+#endif
+
+#ifdef __WIN32__
+midibus::midibus( char a_id, int a_queue )
+{
+    /* set members */
+    m_queue          = a_queue;
     m_id = a_id;
     m_clock_type = e_clock_off;
     m_inputing = false;
@@ -84,34 +115,33 @@ midibus::midibus( int a_localclient,
 
     m_name = tmp;
 }
+#endif
 
 
 int midibus::m_clock_mod = 16 * 4;
 
-void 
+void
 midibus::lock( )
 {
-   m_mutex.lock();
+    m_mutex.lock();
 }
 
 
-void 
+void
 midibus::unlock( )
-{ 
+{
     m_mutex.unlock();
 }
 
 
-
-
-
 bool midibus::init_out( )
 {
+#ifdef HAVE_LIBASOUND
     /* temp return */
     int ret;
 
     /* create ports */
-    ret = snd_seq_create_simple_port(m_seq, 
+    ret = snd_seq_create_simple_port(m_seq,
 				     m_name.c_str(),
 			     	     SND_SEQ_PORT_CAP_NO_EXPORT |
 				     SND_SEQ_PORT_CAP_READ,
@@ -125,26 +155,28 @@ bool midibus::init_out( )
     }
 
     /* connect to */
-    ret = snd_seq_connect_to( m_seq, 
-			      m_local_addr_port, 
-			      m_dest_addr_client, 
+    ret = snd_seq_connect_to( m_seq,
+			      m_local_addr_port,
+			      m_dest_addr_client,
 			      m_dest_addr_port );
     if ( ret < 0 ){
-        printf( "snd_seq_connect_to(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        printf( "snd_seq_connect_to(%d:%d) error\n",
+                m_dest_addr_client, m_dest_addr_port);
         return false;
     }
-
+#endif
     return true;
 }
 
 
 bool midibus::init_out_sub( )
 {
+#ifdef HAVE_LIBASOUND
     /* temp return */
     int ret;
 
     /* create ports */
-    ret = snd_seq_create_simple_port(m_seq, 
+    ret = snd_seq_create_simple_port(m_seq,
 				     m_name.c_str(),
 				     SND_SEQ_PORT_CAP_READ |
                      SND_SEQ_PORT_CAP_SUBS_READ,
@@ -156,7 +188,7 @@ bool midibus::init_out_sub( )
         printf( "snd_seq_create_simple_port(write) error\n");
         return false;
     }
-
+#endif
     return true;
 }
 
@@ -164,12 +196,12 @@ bool midibus::init_out_sub( )
 
 bool midibus::init_in( )
 {
-
+#ifdef HAVE_LIBASOUND
     /* temp return */
     int ret;
 
     /* create ports */
-    ret = snd_seq_create_simple_port(m_seq, 
+    ret = snd_seq_create_simple_port(m_seq,
                                      "seq24 in",
                                      SND_SEQ_PORT_CAP_NO_EXPORT |
                                      SND_SEQ_PORT_CAP_WRITE,
@@ -187,8 +219,8 @@ bool midibus::init_in( )
     snd_seq_addr_t sender, dest;
 
     /* the destinatino port is actually our local port */
-    sender.client = m_dest_addr_client; 
-    sender.port = m_dest_addr_port;   
+    sender.client = m_dest_addr_client;
+    sender.port = m_dest_addr_port;
     dest.client = m_local_addr_client;
     dest.port = m_local_addr_port;
 
@@ -204,52 +236,51 @@ bool midibus::init_in( )
     ret = snd_seq_subscribe_port(m_seq, subs);
 
     if ( ret < 0 ){
-        printf( "snd_seq_connect_from(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        printf( "snd_seq_connect_from(%d:%d) error\n",
+                m_dest_addr_client, m_dest_addr_port);
         return false;
     }
-
+#endif
     return true;
 }
 
 
 bool midibus::init_in_sub( )
 {
+#ifdef HAVE_LIBASOUND
     /* temp return */
     int ret;
 
     /* create ports */
-    ret = snd_seq_create_simple_port(m_seq, 
-				     "seq24 in",
-				     SND_SEQ_PORT_CAP_WRITE |
-                     SND_SEQ_PORT_CAP_SUBS_WRITE,
-				     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
-				     SND_SEQ_PORT_TYPE_APPLICATION );
+    ret = snd_seq_create_simple_port(m_seq, "seq24 in",
+            SND_SEQ_PORT_CAP_WRITE |
+            SND_SEQ_PORT_CAP_SUBS_WRITE,
+            SND_SEQ_PORT_TYPE_MIDI_GENERIC |
+            SND_SEQ_PORT_TYPE_APPLICATION );
     m_local_addr_port = ret;
 
     if ( ret < 0 ){
         printf( "snd_seq_create_simple_port(write) error\n");
         return false;
     }
-
+#endif
     return true;
 }
 
 
-
-
 bool midibus::deinit_in( )
 {
-
+#ifdef HAVE_LIBASOUND
     /* temp return */
     int ret;
-    
+
     snd_seq_port_subscribe_t *subs;
     snd_seq_port_subscribe_alloca(&subs);
     snd_seq_addr_t sender, dest;
 
     /* the destinatino port is actually our local port */
-    sender.client = m_dest_addr_client; 
-    sender.port = m_dest_addr_port;   
+    sender.client = m_dest_addr_client;
+    sender.port = m_dest_addr_port;
     dest.client = m_local_addr_client;
     dest.port = m_local_addr_port;
 
@@ -265,10 +296,11 @@ bool midibus::deinit_in( )
     ret = snd_seq_unsubscribe_port(m_seq, subs);
 
     if ( ret < 0 ){
-        printf( "snd_seq_unsubscribe_port(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        printf( "snd_seq_unsubscribe_port(%d:%d) error\n",
+                m_dest_addr_client, m_dest_addr_port);
         return false;
     }
-
+#endif
     return true;
 }
 
@@ -280,7 +312,7 @@ midibus::get_id( )
 }
 
 
-void 
+void
 midibus::print()
 {
     printf( "%s" , m_name.c_str() );
@@ -298,76 +330,67 @@ midibus::~midibus()
 }
 
 
-/* takes an native event, encodes to alsa event, 
+/* takes an native event, encodes to alsa event,
    puts it in the queue */
-void 
+void
 midibus::play( event *a_e24, unsigned char a_channel )
 {
     lock();
 
-  
+#ifdef HAVE_LIBASOUND
 
 		snd_seq_event_t ev;
-		
+
 		/* alsa midi parser */
 		snd_midi_event_t *midi_ev;
-		
+
 		/* temp for midi data */
 		unsigned char buffer[3];
-		
+
 		/* fill buffer and set midi channel */
 		buffer[0] = a_e24->get_status();
 		buffer[0] += (a_channel & 0x0F);
-		
 		a_e24->get_data( &buffer[1], &buffer[2] );
-		
 		snd_midi_event_new( 10, &midi_ev );
-		
+
 		/* clear event */
 		snd_seq_ev_clear( &ev );
-		snd_midi_event_encode( midi_ev,
-							   buffer,
-							   3,
-							   &ev ); 
-		
+		snd_midi_event_encode( midi_ev, buffer, 3, &ev );
 		snd_midi_event_free( midi_ev );
-		
+
 		/* set source */
 		snd_seq_ev_set_source(&ev, m_local_addr_port );
 		snd_seq_ev_set_subs(&ev);
-		
+
 		/* set tag unique to each sequence for removal purposes */
 		//ev.tag = a_tag;
-		
-		// its immediate 
+
+		// its immediate
 		snd_seq_ev_set_direct( &ev );
-		
+
 		/* pump it into the queue */
 		snd_seq_event_output(m_seq, &ev);
-	
-
+#endif
     unlock();
 }
 
 
-inline long 
+inline long
 min ( long a, long b ){
-
-  if ( a < b ) 
-    return a;
-  return b;
-
+    if ( a < b )
+        return a;
+    return b;
 }
 
-/* takes an native event, encodes to alsa event, 
+/* takes an native event, encodes to alsa event,
    puts it in the queue */
-void 
+void
 midibus::sysex( event *a_e24 )
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     snd_seq_event_t ev;
-   
+
     /* clear event */
     snd_seq_ev_clear( &ev );
     snd_seq_ev_set_priority( &ev, 1 );
@@ -375,56 +398,53 @@ midibus::sysex( event *a_e24 )
     /* set source */
     snd_seq_ev_set_source(&ev, m_local_addr_port );
     snd_seq_ev_set_subs(&ev);
-    
-    // its immediate 
+
+    // its immediate
     snd_seq_ev_set_direct( &ev );
 
-    
     unsigned char *data = a_e24->get_sysex();
     long data_size =  a_e24->get_size();
-    for( long offset = 0;
-	 offset < data_size;
-	 offset += c_midibus_sysex_chunk ){
 
-      long data_left = data_size - offset;
+    for (long offset = 0; offset < data_size;
+            offset += c_midibus_sysex_chunk) {
 
-      snd_seq_ev_set_sysex( &ev, 
-			       min( data_left, c_midibus_sysex_chunk), 
-			       &data[offset] ); 
-      
-      /* pump it into the queue */
-      snd_seq_event_output_direct(m_seq, &ev);
+        long data_left = data_size - offset;
 
-      usleep(80000);
-      
-      flush();
+        snd_seq_ev_set_sysex( &ev,
+                min( data_left, c_midibus_sysex_chunk),
+                &data[offset] );
 
+        /* pump it into the queue */
+        snd_seq_event_output_direct(m_seq, &ev);
+        usleep(80000);
+        flush();
     }
-
+#endif
     unlock();
 }
 
 
 // flushes our local queue events out into ALSA
-void 
+void
 midibus::flush()
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     snd_seq_drain_output( m_seq );
-
+#endif
     unlock();
-} 
+}
 
 
-void 
+void
 midibus::init_clock( long a_tick )
 {
+#ifdef HAVE_LIBASOUND
     if ( m_clock_type == e_clock_pos && a_tick != 0)
     {
         continue_from( a_tick );
     }
-    else 
+    else
     if ( m_clock_type == e_clock_mod || a_tick == 0)
     {
         start();
@@ -440,31 +460,31 @@ midibus::init_clock( long a_tick )
         }
         //printf ( "continue_from leftover[%ld] starting_tick[%ld]\n", leftover, starting_tick );
 
-        m_lasttick = starting_tick - 1;    
+        m_lasttick = starting_tick - 1;
 
 
     }
+#endif
 }
 
 void
 midibus::continue_from( long a_tick )
 {
+#ifdef HAVE_LIBASOUND
     /* tell the device that we are going to start at a certain position */
     long pp16th = (c_ppqn / 4);
-
     long leftover = ( a_tick % pp16th );
     long beats = ( a_tick / pp16th );
-
     long starting_tick = a_tick - leftover;
 
-    /* was there anything left?, then wait for next beat (16th note) to start clocking */
+    /* was there anything left? Then wait for next beat (16th note) to start clocking */
     if ( leftover > 0)
     {
         starting_tick += pp16th;
     }
     //printf ( "continue_from leftover[%ld] starting_tick[%ld]\n", leftover, starting_tick );
 
-    m_lasttick = starting_tick - 1;    
+    m_lasttick = starting_tick - 1;
 
     if ( m_clock_type != e_clock_off )
     {
@@ -476,7 +496,7 @@ midibus::continue_from( long a_tick )
         ev.type = SND_SEQ_EVENT_CONTINUE;
         evc.type = SND_SEQ_EVENT_SONGPOS;
         evc.data.control.value = beats;
-        snd_seq_ev_set_fixed( &ev );    
+        snd_seq_ev_set_fixed( &ev );
         snd_seq_ev_set_fixed( &evc );
 
         snd_seq_ev_set_priority( &ev, 1 );
@@ -488,7 +508,7 @@ midibus::continue_from( long a_tick )
         snd_seq_ev_set_source(&ev, m_local_addr_port );
         snd_seq_ev_set_subs(&ev);
 
-        // its immediate 
+        // its immediate
         snd_seq_ev_set_direct( &ev );
         snd_seq_ev_set_direct( &evc );
 
@@ -497,39 +517,40 @@ midibus::continue_from( long a_tick )
         flush();
         snd_seq_event_output(m_seq, &ev);
     }
+#endif
 }
 
 
 /* gets it a runnin */
-void 
+void
 midibus::start()
 {
+#ifdef HAVE_LIBASOUND
     m_lasttick = -1;
-    
-    if ( m_clock_type != e_clock_off ){ 
-	
+
+    if ( m_clock_type != e_clock_off ){
+
 	snd_seq_event_t ev;
-	
 	ev.type = SND_SEQ_EVENT_START;
 	snd_seq_ev_set_fixed( &ev );
-	
 	snd_seq_ev_set_priority( &ev, 1 );
-	
+
 	/* set source */
 	snd_seq_ev_set_source(&ev, m_local_addr_port );
 	snd_seq_ev_set_subs(&ev);
-	
-	// its immediate 
+
+	// its immediate
 	snd_seq_ev_set_direct( &ev );
-	
+
 	/* pump it into the queue */
 	snd_seq_event_output(m_seq, &ev);
 
     }
+#endif
 }
 
 
-void 
+void
 midibus::set_clock( clock_e a_clock_type )
 {
     m_clock_type = a_clock_type;
@@ -543,9 +564,7 @@ midibus::get_clock( )
 }
 
 
-
-
-void 
+void
 midibus::set_input( bool a_inputing )
 {
     if ( m_inputing != a_inputing ){
@@ -559,94 +578,85 @@ midibus::set_input( bool a_inputing )
         {
             deinit_in();
         }
-    }    
+    }
 }
 
 
-bool 
+bool
 midibus::get_input( )
 {
     return m_inputing;
 }
 
 
-
-void 
+void
 midibus::stop()
 {
-
+#ifdef HAVE_LIBASOUND
     m_lasttick = -1;
 
-    if ( m_clock_type != e_clock_off ){   
-	
+    if ( m_clock_type != e_clock_off ){
+
 	snd_seq_event_t ev;
-	
 	ev.type = SND_SEQ_EVENT_STOP;
 	snd_seq_ev_set_fixed( &ev );
-	
 	snd_seq_ev_set_priority( &ev, 1 );
-	
+
 	/* set source */
 	snd_seq_ev_set_source(&ev, m_local_addr_port );
 	snd_seq_ev_set_subs(&ev);
-	
-	// its immediate 
+
+	// its immediate
 	snd_seq_ev_set_direct( &ev );
-	
+
 	/* pump it into the queue */
 	snd_seq_event_output(m_seq, &ev);
-	
     }
+#endif
 }
 
 
-
-
 // generates midi clock
 void
 midibus::clock( long a_tick )
 {
-
     lock();
-
+#ifdef HAVE_LIBASOUND
     if ( m_clock_type != e_clock_off ){
 
 	bool done = false;
-	
 	long uptotick = a_tick;
-	
+
 	if ( m_lasttick >= uptotick )
 	    done = true;
-	
+
 	while ( !done ){
-	    
+
 	    m_lasttick++;
-	    
+
 	    if ( m_lasttick >= uptotick )
 		done = true;
-	    
+
 	    /* tick time? */
 	    if ( m_lasttick % ( c_ppqn / 24 ) == 0 ){
-		
+
 		snd_seq_event_t ev;
-		
 		ev.type = SND_SEQ_EVENT_CLOCK;
-		
+
 		/* set tag to 127 so the sequences
 		   wont remove it */
 		ev.tag = 127;
-		
+
 		snd_seq_ev_set_fixed( &ev );
-		
 		snd_seq_ev_set_priority( &ev, 1 );
-		
+
 		/* set source */
 		snd_seq_ev_set_source(&ev, m_local_addr_port );
 		snd_seq_ev_set_subs(&ev);
-		
-		// its immediate 
+
+		// its immediate
 		snd_seq_ev_set_direct( &ev );
-		
+
 		/* pump it into the queue */
 		snd_seq_event_output(m_seq, &ev);
 
@@ -655,12 +665,12 @@ midibus::clock( long a_tick )
 	/* and send out */
 	flush();
     }
-
+#endif
     unlock();
 }
 
 /* deletes events in queue */
-/*void 
+/*void
 midibus::remove_queued_on_events( int a_tag )
 {
     lock();
@@ -668,8 +678,8 @@ midibus::remove_queued_on_events( int a_tag )
     snd_seq_remove_events_t *remove_events;
 
     snd_seq_remove_events_malloc( &remove_events );
-    
-    snd_seq_remove_events_set_condition( remove_events, 
+
+    snd_seq_remove_events_set_condition( remove_events,
 					 SND_SEQ_REMOVE_OUTPUT |
 					 SND_SEQ_REMOVE_TAG_MATCH |
 					 SND_SEQ_REMOVE_IGNORE_OFF );
@@ -684,7 +694,7 @@ midibus::remove_queued_on_events( int a_tag )
 */
 
 
-void 
+void
 mastermidibus::lock( )
 {
    // printf( "mastermidibus::lock()\n" );
@@ -692,27 +702,27 @@ mastermidibus::lock( )
 }
 
 
-void 
+void
 mastermidibus::unlock( )
-{   
+{
    // printf( "mastermidibus::unlock()\n" );
    m_mutex.unlock();
 }
 
 
 
-/* gets it a runnin */
-void 
+/* gets it running */
+void
 mastermidibus::start()
 {
     lock();
-         
+#ifdef HAVE_LIBASOUND
     /* start timer */
     snd_seq_start_queue( m_alsa_seq, m_queue, NULL );
-    
+
     for ( int i=0; i < m_num_out_buses; i++ )
 	m_buses_out[i]->start();
-
+#endif
      unlock();
 }
 
@@ -722,13 +732,13 @@ mastermidibus::start()
 mastermidibus::continue_from( long a_tick)
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     /* start timer */
     snd_seq_start_queue( m_alsa_seq, m_queue, NULL );
 
     for ( int i=0; i < m_num_out_buses; i++ )
         m_buses_out[i]->continue_from( a_tick );
-
+#endif
     unlock();
 }
 
@@ -741,9 +751,9 @@ mastermidibus::init_clock( long a_tick )
         m_buses_out[i]->init_clock( a_tick );
 
     unlock();
-}    
+}
 
-void 
+void
 mastermidibus::stop()
 {
     lock();
@@ -752,11 +762,13 @@ mastermidibus::stop()
         m_buses_out[i]->stop();
 
 
+#ifdef HAVE_LIBASOUND
     snd_seq_drain_output( m_alsa_seq );
     snd_seq_sync_output_queue( m_alsa_seq );
 
     /* start timer */
     snd_seq_stop_queue( m_alsa_seq, m_queue, NULL );
+#endif
     unlock();
 }
 
@@ -766,20 +778,20 @@ void
 mastermidibus::clock( long a_tick )
 {
     lock();
-    
+
     for ( int i=0; i < m_num_out_buses; i++ )
 	m_buses_out[i]->clock( a_tick );
-    
+
     unlock();
 }
 
-void 
+void
 mastermidibus::set_ppqn( int a_ppqn )
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     m_ppqn = a_ppqn;
-    
+
     /* allocate tempo struct */
     snd_seq_queue_tempo_t *tempo;
     snd_seq_queue_tempo_alloca( &tempo );
@@ -792,16 +804,16 @@ mastermidibus::set_ppqn( int a_ppqn )
 
     /* give tempo struct to the queue */
     snd_seq_set_queue_tempo( m_alsa_seq, m_queue, tempo );
-
+#endif
     unlock();
 }
 
 
-void 
+void
 mastermidibus::set_bpm( int a_bpm )
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     m_bpm = a_bpm;
 
     /* allocate tempo struct */
@@ -815,21 +827,20 @@ mastermidibus::set_bpm( int a_bpm )
 
     /* give tempo struct to the queue */
     snd_seq_set_queue_tempo(m_alsa_seq, m_queue, tempo );
-
+#endif
     unlock();
 }
 
 // flushes our local queue events out into ALSA
-void 
+void
 mastermidibus::flush()
 {
     lock();
-
+#ifdef HAVE_LIBASOUND
     snd_seq_drain_output( m_alsa_seq );
-
+#endif
     unlock();
-} 
-
+}
 
 
 /* fills the array with our buses */
@@ -837,24 +848,25 @@ mastermidibus::mastermidibus()
 {
     /* temp return */
     int ret;
-    
+
     /* set initial number buses */
     m_num_out_buses = 0;
     m_num_in_buses = 0;
 
     for( int i=0; i<c_maxBuses; ++i ){
-        m_buses_in_active[i] = false;	
+        m_buses_in_active[i] = false;
         m_buses_out_active[i] = false;
-        m_buses_in_init[i] = false;	
+        m_buses_in_init[i] = false;
         m_buses_out_init[i] = false;
 
         m_init_clock[i] = e_clock_off;
         m_init_input[i] = false;
     }
-    
+
+#ifdef HAVE_LIBASOUND
     /* open the sequencer client */
     ret = snd_seq_open(&m_alsa_seq, "default",  SND_SEQ_OPEN_DUPLEX, 0);
-    
+
     if ( ret < 0 ){
 	printf( "snd_seq_open() error\n");
 	exit(1);
@@ -865,15 +877,18 @@ mastermidibus::mastermidibus()
 
     /* set up our clients queue */
     m_queue = snd_seq_alloc_queue( m_alsa_seq );
-
+#ifdef LASH_SUPPORT
 	/* notify lash of our client ID so it can restore connections */
 	lash_driver->set_alsa_client_id(snd_seq_client_id(m_alsa_seq));
+#endif
+#endif
 }
 
+
 void
 mastermidibus::init( )
 {
-    
+#ifdef HAVE_LIBASOUND
     /* client info */
     snd_seq_client_info_t *cinfo;
     /* port info */
@@ -885,39 +900,39 @@ mastermidibus::init( )
     snd_seq_client_info_set_client(cinfo, -1);
 
     //printf( "global_ports %d\n", global_manual_alsa_ports );
-    
+
     if ( global_manual_alsa_ports )
     {
 
         int num_buses = 16;
-        
+
         for( int i=0; i<num_buses; ++i )
         {
 
-            m_buses_out[i] = 
+            m_buses_out[i] =
                 new midibus( snd_seq_client_id( m_alsa_seq ), m_alsa_seq, i+1, m_queue );
 
             m_buses_out[i]->init_out_sub();
-            m_buses_out_active[i] = true;		
+            m_buses_out_active[i] = true;
             m_buses_out_init[i] = true;
         }
 
         m_num_out_buses = num_buses;
 
         /* only one in */
-        m_buses_in[0] = 
+        m_buses_in[0] =
             new midibus( snd_seq_client_id( m_alsa_seq ),
                     m_alsa_seq,
                     m_num_in_buses, m_queue);
 
         m_buses_in[0]->init_in_sub();
-        m_buses_in_active[0] = true;	
+        m_buses_in_active[0] = true;
         m_buses_in_init[0] = true;
         m_num_in_buses = 1;
 
 
     }
-    else 
+    else
     {
         /* while the next client one the sequencer is avaiable */
         while (snd_seq_query_next_client(m_alsa_seq, cinfo) >= 0){
@@ -943,7 +958,7 @@ mastermidibus::init( )
                     if ( (cap & SND_SEQ_PORT_CAP_SUBS_WRITE) != 0 &&
                             snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 
-                        m_buses_out[m_num_out_buses] = 
+                        m_buses_out[m_num_out_buses] =
                             new midibus( snd_seq_client_id( m_alsa_seq ),
                                     snd_seq_port_info_get_client(pinfo),
                                     snd_seq_port_info_get_port(pinfo),
@@ -953,20 +968,20 @@ mastermidibus::init( )
                                     m_num_out_buses, m_queue );
 
                         if ( m_buses_out[m_num_out_buses]->init_out() ){
-                            m_buses_out_active[m_num_out_buses] = true;		
+                            m_buses_out_active[m_num_out_buses] = true;
                             m_buses_out_init[m_num_out_buses] = true;
                         } else {
-                            m_buses_out_init[m_num_out_buses] = true;	
+                            m_buses_out_init[m_num_out_buses] = true;
                         }
 
                         m_num_out_buses++;
-                    }	
+                    }
 
                     /* the ins */
                     if ( (cap & SND_SEQ_PORT_CAP_SUBS_READ) != 0 &&
                             snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 
-                        m_buses_in[m_num_in_buses] = 
+                        m_buses_in[m_num_in_buses] =
                             new midibus( snd_seq_client_id( m_alsa_seq ),
                                     snd_seq_port_info_get_client(pinfo),
                                     snd_seq_port_info_get_port(pinfo),
@@ -976,13 +991,13 @@ mastermidibus::init( )
                                     m_num_in_buses, m_queue);
 
                         //if ( m_buses_in[m_num_in_buses]->init_in() ){
-                        m_buses_in_active[m_num_in_buses] = true;	
+                        m_buses_in_active[m_num_in_buses] = true;
                         m_buses_in_init[m_num_in_buses] = true;
                         //} else {
-                        //m_buses_in_init[m_num_in_buses] = true;	
+                        //m_buses_in_init[m_num_in_buses] = true;
                         //}
                         m_num_in_buses++;
-                    }	
+                    }
                 }
             }
 
@@ -1002,27 +1017,27 @@ mastermidibus::init( )
     m_poll_descriptors = new pollfd[m_num_poll_descriptors];
 
     /* get descriptors */
-    snd_seq_poll_descriptors(m_alsa_seq,  
-            m_poll_descriptors, 
-            m_num_poll_descriptors, 
+    snd_seq_poll_descriptors(m_alsa_seq,
+            m_poll_descriptors,
+            m_num_poll_descriptors,
             POLLIN);
 
     set_sequence_input( false, NULL );
 
     /* sizes */
-    snd_seq_set_output_buffer_size(m_alsa_seq, c_midibus_output_size ); 
-    snd_seq_set_input_buffer_size(m_alsa_seq, c_midibus_input_size ); 
+    snd_seq_set_output_buffer_size(m_alsa_seq, c_midibus_output_size );
+    snd_seq_set_input_buffer_size(m_alsa_seq, c_midibus_input_size );
 
 
-    m_bus_announce = 
+    m_bus_announce =
         new midibus( snd_seq_client_id( m_alsa_seq ),
                 SND_SEQ_CLIENT_SYSTEM,
                 SND_SEQ_PORT_SYSTEM_ANNOUNCE,
                 m_alsa_seq,
-                "system","annouce",
+                "system", "annouce",
                 0, m_queue);
 
-    m_bus_announce->set_input(true);   
+    m_bus_announce->set_input(true);
 
 
     for ( int i=0; i<m_num_out_buses; i++ )
@@ -1031,14 +1046,56 @@ mastermidibus::init( )
     for ( int i=0; i<m_num_in_buses; i++ )
         set_input(i,m_init_input[i]);
 
-    
+#endif
+#ifdef __WIN32__
+    int client;
+
+        int num_buses = 16;
+
+        for( int i=0; i<num_buses; ++i )
+        {
+
+            m_buses_out[i] =
+                new midibus( i+1, m_queue );
+
+            m_buses_out[i]->init_out_sub();
+            m_buses_out_active[i] = true;
+            m_buses_out_init[i] = true;
+        }
+
+        m_num_out_buses = num_buses;
+
+        /* only one in */
+        m_buses_in[0] =
+            new midibus( m_num_in_buses, m_queue);
+
+        m_buses_in[0]->init_in_sub();
+        m_buses_in_active[0] = true;
+        m_buses_in_init[0] = true;
+        m_num_in_buses = 1;
+
+
+    set_bpm( c_bpm );
+    set_ppqn( c_ppqn );
+
+    /* midi input */
+    /* poll descriptors */
+
+    set_sequence_input( false, NULL );
+
+    for ( int i=0; i<m_num_out_buses; i++ )
+        set_clock(i,m_init_clock[i]);
+
+    for ( int i=0; i<m_num_in_buses; i++ )
+        set_input(i,m_init_input[i]);
+#endif
 }
-      
+
 mastermidibus::~mastermidibus()
 {
     for ( int i=0; i<m_num_out_buses; i++ )
 	delete m_buses_out[i];
-
+#ifdef HAVE_LIBASOUND
     snd_seq_event_t ev;
 
     /* kill timer */
@@ -1049,12 +1106,12 @@ mastermidibus::~mastermidibus()
 
     /* close client */
     snd_seq_close( m_alsa_seq );
-
+#endif
 }
 
 
 
-void      
+void
 mastermidibus::sysex( event *a_ev )
 {
 	lock();
@@ -1068,7 +1125,7 @@ mastermidibus::sysex( event *a_ev )
 }
 
 
-void 
+void
 mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel )
 {
 	lock();
@@ -1079,7 +1136,7 @@ mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel
 }
 
 
-void 
+void
 mastermidibus::set_clock( unsigned char a_bus, clock_e a_clock_type )
 {
     lock();
@@ -1092,7 +1149,7 @@ mastermidibus::set_clock( unsigned char a_bus, clock_e a_clock_type )
     unlock();
 }
 
-clock_e 
+clock_e
 mastermidibus::get_clock( unsigned char a_bus )
 {
 	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
@@ -1101,35 +1158,35 @@ mastermidibus::get_clock( unsigned char a_bus )
 	return e_clock_off;
 }
 
-void 
+void
 midibus::set_clock_mod( int a_clock_mod )
 {
     if (a_clock_mod != 0 )
         m_clock_mod = a_clock_mod;
 }
 
-int 
-midibus::get_clock_mod( void )
+int
+midibus::get_clock_mod()
 {
     return m_clock_mod;
 }
-      
 
-void 
+
+void
 mastermidibus::set_input( unsigned char a_bus, bool a_inputing )
 {
     lock();
     if ( a_bus < c_maxBuses ){
         m_init_input[a_bus] = a_inputing;
     }
- 
+
     if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
         m_buses_in[a_bus]->set_input( a_inputing );
     }
     unlock();
 }
 
-bool 
+bool
 mastermidibus::get_input( unsigned char a_bus )
 {
 	if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
@@ -1145,7 +1202,7 @@ mastermidibus::get_midi_out_bus_name( int a_bus )
 	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		return m_buses_out[a_bus]->get_name();
 	}
-	
+
 	/* copy names */
 	char tmp[60];
 
@@ -1160,8 +1217,8 @@ mastermidibus::get_midi_out_bus_name( int a_bus )
 	}
 
 	string ret = tmp;
-	return ret;	
-	
+	return ret;
+
 }
 
 
@@ -1171,7 +1228,7 @@ mastermidibus::get_midi_in_bus_name( int a_bus )
 	if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
 		return m_buses_in[a_bus]->get_name();
 	}
-	
+
 	/* copy names */
 	char tmp[60];
 
@@ -1186,29 +1243,29 @@ mastermidibus::get_midi_in_bus_name( int a_bus )
 	}
 
 	string ret = tmp;
-	return ret;	
-	
+	return ret;
+
 }
 
 
-void 
+void
 mastermidibus::print()
 {
     printf( "Available Buses\n");
     for ( int i=0; i<m_num_out_buses; i++ ){
-	printf( "%s\n", m_buses_out[i]->m_name.c_str() ); 
+	printf( "%s\n", m_buses_out[i]->m_name.c_str() );
     }
 }
 
 
-int 
+int
 mastermidibus::get_num_out_buses()
 {
     return m_num_out_buses;
 }
 
 
-int 
+int
 mastermidibus::get_num_in_buses()
 {
     return m_num_in_buses;
@@ -1217,53 +1274,56 @@ mastermidibus::get_num_in_buses()
 int
 mastermidibus::poll_for_midi( )
 {
-    int ret;
-
-    ret = poll( m_poll_descriptors, 
-		 m_num_poll_descriptors, 
+    int ret = 0;
+#ifdef HAVE_LIBASOUND
+    ret = poll( m_poll_descriptors,
+		 m_num_poll_descriptors,
 		 1000);
-
+#endif
     return ret;
 }
 
-bool 
+bool
 mastermidibus::is_more_input( ){
-    
+
     lock();
 
-    int size = snd_seq_event_input_pending(m_alsa_seq, 0);
+    int size=0;
 
+#ifdef HAVE_LIBASOUND
+    size = snd_seq_event_input_pending(m_alsa_seq, 0);
+#endif
     unlock();
 
     return ( size > 0 );
 }
 
 
-void 
+void
 mastermidibus::port_start( int a_client, int a_port )
 {
     lock();
-    
- 
-    
+
+#ifdef HAVE_LIBASOUND
+
     /* client info */
     snd_seq_client_info_t *cinfo;
     snd_seq_client_info_alloca(&cinfo);
-    snd_seq_get_any_client_info( m_alsa_seq, a_client, cinfo );  
-    
+    snd_seq_get_any_client_info( m_alsa_seq, a_client, cinfo );
+
     /* port info */
     snd_seq_port_info_t *pinfo;
-    
+
     /* fill pinfo */
     snd_seq_port_info_alloca(&pinfo);
-    snd_seq_get_any_port_info( m_alsa_seq, a_client, a_port, pinfo );  
-    
-    
+    snd_seq_get_any_port_info( m_alsa_seq, a_client, a_port, pinfo );
+
+
     /* get its capability */
     int cap =  snd_seq_port_info_get_capability(pinfo);
-    
+
     if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-        
+
         /* the outs */
         if ( (cap & (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE ))
              == (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE )
@@ -1272,19 +1332,19 @@ mastermidibus::port_start( int a_client, int a_port )
 
             bool replacement = false;
             int bus_slot = m_num_out_buses;
-            
+
             for( int i=0; i< m_num_out_buses; i++ ){
-                
+
                 if( m_buses_out[i]->get_client() == a_client  &&
                     m_buses_out[i]->get_port() == a_port &&
                     m_buses_out_active[i] == false ){
-                    
+
                     replacement = true;
                     bus_slot = i;
                 }
             }
-            
-            m_buses_out[bus_slot] = 
+
+            m_buses_out[bus_slot] =
                 new midibus( snd_seq_client_id( m_alsa_seq ),
                              snd_seq_port_info_get_client(pinfo),
                              snd_seq_port_info_get_port(pinfo),
@@ -1292,40 +1352,40 @@ mastermidibus::port_start( int a_client, int a_port )
                              snd_seq_client_info_get_name(cinfo),
                              snd_seq_port_info_get_name(pinfo),
                              m_num_out_buses, m_queue );
-            
+
             m_buses_out[bus_slot]->init_out();
             m_buses_out_active[bus_slot] = true;
             m_buses_out_init[bus_slot] = true;
-            
+
             if ( !replacement ){
                 m_num_out_buses++;
             }
-        }	
-        
+        }
+
         /* the ins */
         if ( (cap & (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ ))
              == (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ )
              && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-            
+
             bool replacement = false;
             int bus_slot = m_num_in_buses;
 
-          
-            
+
+
             for( int i=0; i< m_num_in_buses; i++ ){
-                
+
                 if( m_buses_in[i]->get_client() == a_client  &&
                     m_buses_in[i]->get_port() == a_port &&
                     m_buses_in_active[i] == false ){
-                    
+
                     replacement = true;
                     bus_slot = i;
                 }
             }
 
             //printf( "in [%d] [%d]\n", replacement, bus_slot );
-            
-            m_buses_in[bus_slot] = 
+
+            m_buses_in[bus_slot] =
                 new midibus( snd_seq_client_id( m_alsa_seq ),
                              snd_seq_port_info_get_client(pinfo),
                              snd_seq_port_info_get_port(pinfo),
@@ -1333,20 +1393,20 @@ mastermidibus::port_start( int a_client, int a_port )
                              snd_seq_client_info_get_name(cinfo),
                              snd_seq_port_info_get_name(pinfo),
                              m_num_in_buses, m_queue);
-            
+
             //m_buses_in[bus_slot]->init_in();
             m_buses_in_active[bus_slot] = true;
             m_buses_in_init[bus_slot] = true;
 
-            if ( !replacement ){            
+            if ( !replacement ){
                 m_num_in_buses++;
             }
-        }	
+        }
     }
-    
+
     /* end loop for clients */
-    
-    
+
+
     /* midi input */
     /* poll descriptors */
 
@@ -1355,21 +1415,21 @@ mastermidibus::port_start( int a_client, int a_port )
 
     /* allocate into */
     m_poll_descriptors = new pollfd[m_num_poll_descriptors];
-    
+
     /* get descriptors */
-    snd_seq_poll_descriptors(m_alsa_seq,  
-			     m_poll_descriptors, 
-			     m_num_poll_descriptors, 
+    snd_seq_poll_descriptors(m_alsa_seq,
+			     m_poll_descriptors,
+			     m_num_poll_descriptors,
 			     POLLIN);
-    
+#endif
     unlock();
 }
 
-void 
+void
 mastermidibus::port_exit( int a_client, int a_port )
 {
 	lock();
-
+#ifdef HAVE_LIBASOUND
 	for( int i=0; i< m_num_out_buses; i++ ){
 
 		if( m_buses_out[i]->get_client() == a_client  &&
@@ -1387,7 +1447,7 @@ mastermidibus::port_exit( int a_client, int a_port )
 			m_buses_in_active[i] = false;
 		}
 	}
-
+#endif
 	unlock();
 }
 
@@ -1396,124 +1456,119 @@ bool
 mastermidibus::get_midi_event( event *a_in )
 {
     lock();
-    
-    snd_seq_event_t *ev; 
-    
+#ifdef HAVE_LIBASOUND
+    snd_seq_event_t *ev;
+
     bool sysex = false;
-    
+    bool ret = false;
+
     /* temp for midi data */
     unsigned char buffer[0x1000];
-    
+
     snd_seq_event_input(m_alsa_seq, &ev);
-    
-    
-    
-    bool ret = false;
 
     if (! global_manual_alsa_ports )
     {
-        switch( ev->type ){ 
+        switch(ev->type) {
 
             case SND_SEQ_EVENT_PORT_START:
-                {   
-                    //printf("SND_SEQ_EVENT_PORT_START:    addr[%d:%d]\n", 
+                {
+                    //printf("SND_SEQ_EVENT_PORT_START:    addr[%d:%d]\n",
                     //	   ev->data.addr.client, ev->data.addr.port );
                     port_start( ev->data.addr.client, ev->data.addr.port );
-                    ret = true; 
+                    ret = true;
                     break;
                 }
 
-            case SND_SEQ_EVENT_PORT_EXIT:     
+            case SND_SEQ_EVENT_PORT_EXIT:
                 {
-                    //printf("SND_SEQ_EVENT_PORT_EXIT:     addr[%d:%d]\n", 
-                    //	   ev->data.addr.client, ev->data.addr.port ); 
+                    //printf("SND_SEQ_EVENT_PORT_EXIT:     addr[%d:%d]\n",
+                    //	   ev->data.addr.client, ev->data.addr.port );
                     port_exit( ev->data.addr.client, ev->data.addr.port );
-                    ret = true; 
+                    ret = true;
                     break;
                 }
 
             case SND_SEQ_EVENT_PORT_CHANGE:
-                {   
-                    //printf("SND_SEQ_EVENT_PORT_CHANGE:   addr[%d:%d]\n", 
-                    //	   ev->data.addr.client, 
-                    //	   ev->data.addr.port ); 
-                    ret = true; 
+                {
+                    //printf("SND_SEQ_EVENT_PORT_CHANGE:   addr[%d:%d]\n",
+                    //	   ev->data.addr.client,
+                    //	   ev->data.addr.port );
+                    ret = true;
                     break;
                 }
 
-            default: break;
+            default:
+                break;
 
         }
     }
 
-    if( ret ){
+    if (ret) {
         unlock();
         return false;
     }
-    
+
     /* alsa midi parser */
     snd_midi_event_t *midi_ev;
-    snd_midi_event_new( 0x1000, &midi_ev );
-    
-    long bytes =
-        snd_midi_event_decode( midi_ev,
-                               buffer,
-                               0x1000,
-                               ev ); 
-    
+    snd_midi_event_new(sizeof(buffer), &midi_ev);
+
+    long bytes = snd_midi_event_decode(midi_ev, buffer, sizeof(buffer), ev);
+
+    if (bytes <= 0) {
+        unlock();
+        return false;
+    }
     a_in->set_timestamp( ev->time.tick );
     a_in->set_status( buffer[0] );
     a_in->set_size( bytes );
-    
-    /* we will only get EVENT_SYSEX on the first 
+
+    /* we will only get EVENT_SYSEX on the first
        packet of midi data, the rest we have
        to poll for */
     //if ( buffer[0] == EVENT_SYSEX ){
-    if ( 0 ){
-    
+    if (0) {
+
         /* set up for sysex if needed */
         a_in->start_sysex( );
         sysex = a_in->append_sysex( buffer, bytes );
     }
     else {
-        
         a_in->set_data( buffer[1], buffer[2] );
-        
+
         // some keyboards send on's with vel 0 for off
         if ( a_in->get_status() == EVENT_NOTE_ON &&
              a_in->get_note_velocity() == 0x00 ){
             a_in->set_status( EVENT_NOTE_OFF );
         }
-        
+
         sysex = false;
     }
-    
+
     /* sysex messages might be more than one message */
-    while ( sysex ){
-        
+    while (sysex) {
+
         snd_seq_event_input(m_alsa_seq, &ev);
-        
-        bytes =
-            snd_midi_event_decode( midi_ev,
-                                   buffer,
-                                   0x1000,
-                                   ev ); 
-        
-        sysex = a_in->append_sysex( buffer, bytes );
-        
+
+        bytes = snd_midi_event_decode(midi_ev, buffer, sizeof(buffer), ev);
+
+        if (bytes > 0)
+            sysex = a_in->append_sysex( buffer, bytes );
+        else
+            sysex = false;
     }
-    
-    snd_seq_free_event( ev );
+
     snd_midi_event_free( midi_ev );
-    
+
+#endif
+
     unlock();
-    
     return true;
 }
 
-void 
+void
 mastermidibus::set_sequence_input( bool a_state, sequence *a_seq )
-{  
+{
     lock();
 
     m_seq = a_seq;
@@ -1521,4 +1576,4 @@ mastermidibus::set_sequence_input( bool a_state, sequence *a_seq )
 
     unlock();
 }
-  
+
