@@ -18,17 +18,27 @@
 //
 //-----------------------------------------------------------------------------
 
-class midibus;
+#pragma once
+
+/* forward declarations*/
 class mastermidibus;
+class midibus;
 
-#ifndef SEQ24_MIDIBUS
-#define SEQ24_MIDIBUS
+#ifdef __WIN32__
+#   include "configwin32.h"
+#   include "midibus_portmidi.h"
+#else
+#include "config.h"
+
+#if HAVE_LIBASOUND
+#    include <alsa/asoundlib.h>
+#    include <alsa/seq_midi_event.h>
+#endif
+
+#include <string>
 
-#include <alsa/asoundlib.h>
-#include <alsa/seq_midi_event.h>
 #include "event.h"
 #include "sequence.h"
-#include <string>
 #include "mutex.h"
 #include "globals.h"
 
@@ -44,9 +54,10 @@ enum clock_e
 
 };
 
+
 class midibus
 {
-    
+
  private:
 
     int m_id;
@@ -57,14 +68,16 @@ class midibus
     static int m_clock_mod;
 
     /* sequencer client handle */
-    snd_seq_t *m_seq;
+#if HAVE_LIBASOUND
+    snd_seq_t * const m_seq;
 
     /* address of client */
-    int m_dest_addr_client;
-    int m_dest_addr_port;
+    const int m_dest_addr_client;
+    const int m_dest_addr_port;
 
-    int m_local_addr_client;
+    const int m_local_addr_client;
     int m_local_addr_port;
+#endif
 
     /* id of queue */
     int m_queue;
@@ -83,26 +96,32 @@ class midibus
     void lock();
     void unlock();
 
-   
+
 
  public:
 
-    /* constructor, client#, port#, sequencer, 
+#if HAVE_LIBASOUND	
+    /* constructor, client#, port#, sequencer,
        name of client, name of port */
     midibus( int a_localclient,
-	     int a_destclient, 
-	     int a_destport, 
-	     snd_seq_t  *a_seq, 
-	     const char *a_client_name, 
+	     int a_destclient,
+	     int a_destport,
+	     snd_seq_t  *a_seq,
+	     const char *a_client_name,
 	     const char *a_port_name,
 	     int a_id,
          int a_queue );
 
     midibus( int a_localclient,
-	     snd_seq_t  *a_seq, 
+	     snd_seq_t  *a_seq,
 	     int a_id,
          int a_queue );
-    
+#endif
+
+#ifdef __WIN32__
+    midibus( char a_id, int a_queue );
+#endif
+
     ~midibus();
 
     bool init_out(  );
@@ -110,7 +129,7 @@ class midibus
     bool deinit_in(  );
     bool init_out_sub(  );
     bool init_in_sub(  );
- 
+
     void print();
 
     string get_name();
@@ -125,7 +144,7 @@ class midibus
     void start();
     void stop();
     void clock(  long a_tick );
-    void continue_from( long a_tick ); 
+    void continue_from( long a_tick );
     void init_clock( long a_tick );
     void set_clock( clock_e a_clocking );
     clock_e get_clock( );
@@ -140,12 +159,14 @@ class midibus
     friend class mastermidibus;
 
 	/* address of client */
-    int get_client( void ) {  return m_dest_addr_client; };
-    int get_port( void ) { return m_dest_addr_port; };
+#if HAVE_LIBASOUND	
+    int get_client(void) {  return m_dest_addr_client; };
+    int get_port(void) { return m_dest_addr_port; };
+#endif
 
     static void set_clock_mod( int a_clock_mod );
-    static int get_clock_mod( void );
- 
+    static int get_clock_mod();
+
 };
 
 class mastermidibus
@@ -153,7 +174,9 @@ class mastermidibus
  private:
 
     /* sequencer client handle */
+#if HAVE_LIBASOUND	
     snd_seq_t *m_alsa_seq;
+#endif
 
     int m_num_out_buses;
     int m_num_in_buses;
@@ -161,10 +184,10 @@ class mastermidibus
     midibus *m_buses_out[c_maxBuses];
     midibus *m_buses_in[c_maxBuses];
     midibus *m_bus_announce;
-    
+
     bool m_buses_out_active[c_maxBuses];
     bool m_buses_in_active[c_maxBuses];
-    
+
     bool m_buses_out_init[c_maxBuses];
     bool m_buses_in_init[c_maxBuses];
 
@@ -176,7 +199,7 @@ class mastermidibus
 
     int m_ppqn;
     int m_bpm;
-    
+
     int  m_num_poll_descriptors;
     struct pollfd *m_poll_descriptors;
 
@@ -192,7 +215,7 @@ class mastermidibus
     void unlock();
 
  public:
-  
+
     mastermidibus();
     ~mastermidibus();
     //midibus *get_default_bus();
@@ -200,8 +223,10 @@ class mastermidibus
 
 
     void init();
-    
+
+#if HAVE_LIBASOUND	
     snd_seq_t* get_alsa_seq( ) { return m_alsa_seq; };
+#endif
 
     int get_num_out_buses();
     int get_num_in_buses();
@@ -210,38 +235,38 @@ class mastermidibus
     void set_ppqn(int a_ppqn);
     int get_bpm(){ return m_bpm;}
     int get_ppqn(){ return m_ppqn;}
-    
+
     string get_midi_out_bus_name( int a_bus );
     string get_midi_in_bus_name( int a_bus );
-    
+
     void print();
-    void flush(); 
-    
+    void flush();
+
     void start();
     void stop();
-    
+
     void clock(  long a_tick );
-    void continue_from( long a_tick ); 
+    void continue_from( long a_tick );
     void init_clock( long a_tick );
-    
+
     int poll_for_midi( );
     bool is_more_input( );
     bool get_midi_event( event *a_in );
     void set_sequence_input( bool a_state, sequence *a_seq );
-    
+
     bool is_dumping( ) { return m_dumping_input; }
     sequence* get_sequence( ) { return m_seq; }
     void sysex( event *a_event );
-    
+
     void port_start( int a_client, int a_port );
     void port_exit( int a_client, int a_port );
-    
+
     void play( unsigned char a_bus, event *a_e24, unsigned char a_channel );
-    
+
     void set_clock( unsigned char a_bus, clock_e a_clock_type );
     clock_e get_clock( unsigned char a_bus );
 
-   
+
     void set_input( unsigned char a_bus, bool a_inputing );
     bool get_input( unsigned char a_bus );
 
