@@ -18,29 +18,28 @@
 //
 //-----------------------------------------------------------------------------
 
-#include "midifile.h"
 #include <iostream>
+#include "midifile.h"
 
-midifile::midifile (string a_name)
+midifile::midifile(const Glib::ustring& a_name) :
+    m_pos(0),
+    m_name(a_name)
 {
-    m_name = a_name;
-    m_pos = 0;
 }
 
 midifile::~midifile ()
 {
 }
 
-
 unsigned long
 midifile::read_long ()
 {
     unsigned long ret = 0;
 
-    ret += (m_d[m_pos++] << 24);
-    ret += (m_d[m_pos++] << 16);
-    ret += (m_d[m_pos++] << 8);
-    ret += (m_d[m_pos++]);
+    ret += (read_byte() << 24);
+    ret += (read_byte() << 16);
+    ret += (read_byte() << 8);
+    ret += (read_byte());
 
     return ret;
 }
@@ -50,13 +49,19 @@ midifile::read_short ()
 {
     unsigned short ret = 0;
 
-    ret += (m_d[m_pos++] << 8);
-    ret += (m_d[m_pos++]);
+    ret += (read_byte() << 8);
+    ret += (read_byte());
 
     //printf( "read_short 0x%4x\n", ret );
     return ret;
 }
 
+unsigned char
+midifile::read_byte ()
+{
+    return m_d[m_pos++];
+}
+
 unsigned long
 midifile::read_var ()
 {
@@ -64,7 +69,7 @@ midifile::read_var ()
     unsigned char c;
 
     /* while bit #7 is set */
-    while (((c = m_d[m_pos++]) & 0x80) != 0x00)
+    while (((c = read_byte()) & 0x80) != 0x00)
     {
         /* shift ret 7 bits */
         ret <<= 7;
@@ -96,12 +101,16 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
     file.seekg (0, ios::beg);
 
     /* alloc data */
-    m_d = (unsigned char *) new char[file_size];
-    if (m_d == NULL) {
+	try
+	{
+	    m_d.resize(file_size);
+	}
+	catch(std::bad_alloc& ex)
+	{
         fprintf(stderr, "Memory allocation failed\n");
         return false;
     }
-    file.read ((char *) m_d, file_size);
+    file.read ((char *) &m_d[0], file_size);
     file.close ();
 
     /* set position to 0 */
@@ -120,7 +129,6 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
     unsigned short NumTracks;
     unsigned short ppqn;
     unsigned short perf;
-    int c;
 
     /* track name from file */
     char TrackName[256];
@@ -145,14 +153,12 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
     /* magic number 'MThd' */
     if (ID != 0x4D546864) {
         fprintf(stderr, "Invalid MIDI header detected: %8lX\n", ID);
-        delete[]m_d;
         return false;
     }
 
     /* we are only supporting format 1 for now */
     if (Format != 1) {
         fprintf(stderr, "Unsupported MIDI format detected: %d\n", Format);
-        delete[]m_d;
         return false;
     }
 
@@ -183,7 +189,6 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
             seq = new sequence ();
             if (seq == NULL) {
                 fprintf(stderr, "Memory allocation failed\n");
-                delete[]m_d;
                 return false;
             }
             seq->set_master_midi_bus (&a_perf->m_master_bus);
@@ -235,8 +240,8 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                     case EVENT_CONTROL_CHANGE:
                     case EVENT_PITCH_WHEEL:
 
-                        data[0] = m_d[m_pos++];
-                        data[1] = m_d[m_pos++];
+                        data[0] = read_byte();
+                        data[1] = read_byte();
 
                         // some files have vel=0 as note off
                         if ((status & 0xF0) == EVENT_NOTE_ON && data[1] == 0)
@@ -258,7 +263,7 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                     case EVENT_PROGRAM_CHANGE:
                     case EVENT_CHANNEL_PRESSURE:
 
-                        data[0] = m_d[m_pos++];
+                        data[0] = read_byte();
                         //printf( "%02X\n", data[0] );
 
                         /* set data and add */
@@ -275,7 +280,7 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                         if (status == 0xFF)
                         {
                             /* get meta type */
-                            type = m_d[m_pos++];
+                            type = read_byte();
                             len = read_var ();
 
                             //printf( "%02X %08X ", type, (int) len );
@@ -294,20 +299,20 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
 
                                     if (proprietary == c_midibus)
                                     {
-                                        seq->set_midi_bus (m_d[m_pos++]);
+                                        seq->set_midi_bus (read_byte());
                                         len--;
                                     }
 
                                     else if (proprietary == c_midich)
                                     {
-                                        seq->set_midi_channel (m_d[m_pos++]);
+                                        seq->set_midi_channel (read_byte());
                                         len--;
                                     }
 
                                     else if (proprietary == c_timesig)
                                     {
-                                        seq->set_bpm (m_d[m_pos++]);
-                                        seq->set_bw (m_d[m_pos++]);
+                                        seq->set_bpm (read_byte());
+                                        seq->set_bw (read_byte());
                                         len -= 2;
                                     }
 
@@ -351,8 +356,8 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                                     /* Trk Done */
                                 case 0x2f:
 
-                                    // If delta is 0, then another event happened at the same time 
-                                    // as the track end.  the sequence class will discard the last 
+                                    // If delta is 0, then another event happened at the same time
+                                    // as the track end.  the sequence class will discard the last
                                     // note.  This is a fix for that.   Native Seq24 file will always
                                     // have a Delta >= 1
                                     if ( Delta == 0 ){
@@ -368,7 +373,7 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                                 case 0x03:
                                     for (i = 0; i < len; i++)
                                     {
-                                        TrackName[i] = m_d[m_pos++];
+                                        TrackName[i] = read_byte();
                                     }
 
                                     TrackName[i] = '\0';
@@ -390,20 +395,24 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                                 default:
                                     for (i = 0; i < len; i++)
                                     {
-                                        c = m_d[m_pos++];
-                                        //printf( "%02X ", c  );  
+                                        read_byte();
                                     }
-                                    //printf("\n");
                                     break;
                             }
                         }
-                        else
+                        else if(status == 0xF0)
                         {
                             /* sysex */
-                            fprintf(stderr,
-                                    "No support for SYSEX messages: %hhu\n",
-                                    status);
-                            delete[]m_d;
+                            len = read_var ();
+
+                            /* skip it */
+                            m_pos += len;
+
+                            fprintf(stderr, "Warning, no support for SYSEX messages, discarding.\n");
+                        }
+                        else
+                        {
+                            fprintf(stderr, "Unexpected system event : 0x%.2X", status);
                             return false;
                         }
 
@@ -411,7 +420,6 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
 
                     default:
                         fprintf(stderr, "Unsupported MIDI event: %hhu\n", status);
-                        delete[]m_d;
                         return false;
                         break;
                 }
@@ -419,14 +427,14 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
             }			/* while ( !done loading Trk chunk */
 
             /* the sequence has been filled, add it  */
-            //printf ( "add_sequence( %d )\n", perf + (a_screen_set * c_seqs_in_set)); 
+            //printf ( "add_sequence( %d )\n", perf + (a_screen_set * c_seqs_in_set));
             a_perf->add_sequence (seq, perf + (a_screen_set * c_seqs_in_set));
         }
 
         /* dont know what kind of chunk */
         else
         {
-            /* its not a MTrk, we dont know how to deal with it, 
+            /* its not a MTrk, we dont know how to deal with it,
                so we just eat it */
             fprintf(stderr, "Unsupported MIDI header detected: %8lX\n", ID);
             m_pos += TrackLength;
@@ -447,29 +455,29 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
             for (unsigned int i = 0; i < seqs; i++)
             {
 
-                a_perf->get_midi_control_toggle (i)->m_active = m_d[m_pos++];
+                a_perf->get_midi_control_toggle (i)->m_active = read_byte();
                 a_perf->get_midi_control_toggle (i)->m_inverse_active =
-                    m_d[m_pos++];
-                a_perf->get_midi_control_toggle (i)->m_status = m_d[m_pos++];
-                a_perf->get_midi_control_toggle (i)->m_data = m_d[m_pos++];
-                a_perf->get_midi_control_toggle (i)->m_min_value = m_d[m_pos++];
-                a_perf->get_midi_control_toggle (i)->m_max_value = m_d[m_pos++];
+                    read_byte();
+                a_perf->get_midi_control_toggle (i)->m_status = read_byte();
+                a_perf->get_midi_control_toggle (i)->m_data = read_byte();
+                a_perf->get_midi_control_toggle (i)->m_min_value = read_byte();
+                a_perf->get_midi_control_toggle (i)->m_max_value = read_byte();
 
-                a_perf->get_midi_control_on (i)->m_active = m_d[m_pos++];
+                a_perf->get_midi_control_on (i)->m_active = read_byte();
                 a_perf->get_midi_control_on (i)->m_inverse_active =
-                    m_d[m_pos++];
-                a_perf->get_midi_control_on (i)->m_status = m_d[m_pos++];
-                a_perf->get_midi_control_on (i)->m_data = m_d[m_pos++];
-                a_perf->get_midi_control_on (i)->m_min_value = m_d[m_pos++];
-                a_perf->get_midi_control_on (i)->m_max_value = m_d[m_pos++];
+                    read_byte();
+                a_perf->get_midi_control_on (i)->m_status = read_byte();
+                a_perf->get_midi_control_on (i)->m_data = read_byte();
+                a_perf->get_midi_control_on (i)->m_min_value = read_byte();
+                a_perf->get_midi_control_on (i)->m_max_value = read_byte();
 
-                a_perf->get_midi_control_off (i)->m_active = m_d[m_pos++];
+                a_perf->get_midi_control_off (i)->m_active = read_byte();
                 a_perf->get_midi_control_off (i)->m_inverse_active =
-                    m_d[m_pos++];
-                a_perf->get_midi_control_off (i)->m_status = m_d[m_pos++];
-                a_perf->get_midi_control_off (i)->m_data = m_d[m_pos++];
-                a_perf->get_midi_control_off (i)->m_min_value = m_d[m_pos++];
-                a_perf->get_midi_control_off (i)->m_max_value = m_d[m_pos++];
+                    read_byte();
+                a_perf->get_midi_control_off (i)->m_status = read_byte();
+                a_perf->get_midi_control_off (i)->m_data = read_byte();
+                a_perf->get_midi_control_off (i)->m_min_value = read_byte();
+                a_perf->get_midi_control_off (i)->m_max_value = read_byte();
             }
         }
 
@@ -481,7 +489,7 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
             /* TrackLength is nyumber of buses */
             for (unsigned int x = 0; x < TrackLength; x++)
             {
-                int bus_on = m_d[m_pos++];
+                int bus_on = read_byte();
                 a_perf->get_master_midi_bus ()->set_clock (x, (clock_e) bus_on);
             }
         }
@@ -499,15 +507,12 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
             {
                 /* get the length of the string */
                 unsigned int len = read_short ();
-                char * notes = new char[len + 1];
+                string notess;
 
                 for (unsigned int i = 0; i < len; i++)
-                    notes[i] = m_d[m_pos++];
+                    notess += read_byte();
 
-                notes[len] = '\0';
-                string notess (notes);
                 a_perf->set_screen_set_notepad (x, &notess);
-                delete[]notes;
             }
         }
     }
@@ -523,9 +528,29 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
         }
     }
 
+    // read in the mute group info.
+    if ((file_size - m_pos) > (int) sizeof (unsigned long))
+    {
+        ID = read_long ();
+        if (ID == c_mutegroups)
+        {
+            long length = read_long ();
+            if (c_gmute_tracks != length)
+            {
+                printf( "corrupt data in mutegroup section\n" );
+            }
+            for (int i = 0; i < c_seqs_in_set; i++)
+            {
+                a_perf->select_group_mute(read_long ());
+                for (int k = 0; k < c_seqs_in_set; ++k) {
+                    a_perf->set_group_mute_state(k, read_long ());
+                }
+            }
+        }
+    }
+
     // *** ADD NEW TAGS AT END **************/
 
-    delete[]m_d;
     return true;
     //printf ( "done\n");
 }
@@ -534,40 +559,32 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
 void
 midifile::write_long (unsigned long a_x)
 {
-    m_l.push_front ((a_x & 0xFF000000) >> 24);
-    m_l.push_front ((a_x & 0x00FF0000) >> 16);
-    m_l.push_front ((a_x & 0x0000FF00) >> 8);
-    m_l.push_front ((a_x & 0x000000FF));
+    write_byte ((a_x & 0xFF000000) >> 24);
+    write_byte ((a_x & 0x00FF0000) >> 16);
+    write_byte ((a_x & 0x0000FF00) >> 8);
+    write_byte ((a_x & 0x000000FF));
 }
 
 
 void
 midifile::write_short (unsigned short a_x)
 {
-    m_l.push_front ((a_x & 0xFF00) >> 8);
-    m_l.push_front ((a_x & 0x00FF));
+    write_byte ((a_x & 0xFF00) >> 8);
+    write_byte ((a_x & 0x00FF));
+}
+
+void
+midifile::write_byte (unsigned char a_x)
+{
+    m_l.push_back (a_x);
 }
 
 bool midifile::write (perform * a_perf)
 {
-    /* open binary file */
-    ofstream file (m_name.c_str (), ios::out | ios::binary | ios::trunc);
-
-    if (!file.is_open ())
-        return false;
-
-    /* used in small loops */
-    int i;
-
-    /* sequence pointer */
-    sequence * seq;
-    event e;
-    list<char> l;
-
     int numtracks = 0;
 
     /* get number of tracks */
-    for (i = 0; i < c_max_sequence; i++)
+    for (int i = 0; i < c_max_sequence; i++)
     {
         if (a_perf->is_active (i))
             numtracks++;
@@ -589,13 +606,13 @@ bool midifile::write (perform * a_perf)
     /* for each Track in the midi file */
     for (int curTrack = 0; curTrack < c_max_sequence; curTrack++)
     {
-
         if (a_perf->is_active (curTrack))
         {
+            /* sequence pointer */
+            sequence * seq = a_perf->get_sequence (curTrack);
 
             //printf ("track[%d]\n", curTrack );
-
-            seq = a_perf->get_sequence (curTrack);
+            list<char> l;
             seq->fill_list (&l, curTrack);
 
             /* magic number 'MTrk' */
@@ -606,7 +623,7 @@ bool midifile::write (perform * a_perf)
 
             while (l.size () > 0)
             {
-                m_l.push_front (l.back ());
+                write_byte (l.back ());
                 l.pop_back ();
             }
         }
@@ -627,13 +644,13 @@ bool midifile::write (perform * a_perf)
     write_long (c_notes);
     write_short (c_max_sets);
 
-    for (i = 0; i < c_max_sets; i++)
+    for (int i = 0; i < c_max_sets; i++)
     {
         string * note = a_perf->get_screen_set_notepad (i);
         write_short (note->length ());
 
         for (unsigned int j = 0; j < note->length (); j++)
-            m_l.push_front ((*note)[j]);
+            write_byte ((*note)[j]);
     }
 
 
@@ -641,29 +658,38 @@ bool midifile::write (perform * a_perf)
     write_long (c_bpmtag);
     write_long (a_perf->get_bpm ());
 
-    int data_size = m_l.size ();
-    m_d = (unsigned char *) new char[data_size];
-
-    m_pos = 0;
-
-    for (list < unsigned char >::reverse_iterator ri = m_l.rbegin ();
-            ri != m_l.rend (); ri++)
+    /* write out the mute groups */
+    write_long (c_mutegroups);
+    write_long (c_gmute_tracks);
+    for (int j=0; j < c_seqs_in_set; ++j)
     {
-        m_d[m_pos++] = *ri;
+        a_perf->select_group_mute(j);
+        write_long(j);
+        for (int i=0; i < c_seqs_in_set; ++i)
+        {
+            write_long( a_perf->get_group_mute_state(i) );
+        }
+    }
+
+    /* open binary file */
+    ofstream file (m_name.c_str (), ios::out | ios::binary | ios::trunc);
+
+    if (!file.is_open ())
+        return false;
+
+    /* enable bufferization */
+    char file_buffer[1024];
+    file.rdbuf()->pubsetbuf(file_buffer, sizeof file_buffer);
+
+    for (list < unsigned char >::iterator i = m_l.begin ();
+            i != m_l.end (); i++)
+    {
+      char c = *i;
+      file.write(&c, 1);
     }
 
     m_l.clear ();
 
-    // super slow
-    //while ( m_l.size() > 0 ){
-    //m_d[m_pos++] =  m_l.back();
-    //  m_l.pop_back();
-    //}
-
-    file.write ((char *) m_d, data_size);
-    file.close ();
-
-    delete[]m_d;
-
     return true;
 }
+
