@@ -19,7 +19,6 @@
 //-----------------------------------------------------------------------------
 
 #include "mutex.h"
-#include "config.h"
 
 const pthread_mutex_t mutex::recmutex = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
 const pthread_cond_t condition_var::cond  = PTHREAD_COND_INITIALIZER;
@@ -29,16 +28,16 @@ mutex::mutex( )
     m_mutex_lock = recmutex;
 }
 
-void 
+void
 mutex::lock( )
 {
     pthread_mutex_lock( &m_mutex_lock );
 }
 
 
-void 
+void
 mutex::unlock( )
-{      
+{
     pthread_mutex_unlock( &m_mutex_lock );
 }
 
