@@ -17,26 +17,26 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 //-----------------------------------------------------------------------------
+
+#pragma once
+
 #include "globals.h"
-#include "config.h"
+
 #include <pthread.h>
 
-#ifndef SEQ24_MUTEX
-#define SEQ24_MUTEX
-
 class mutex {
-    
+
 private:
 
     static const pthread_mutex_t recmutex;
-    
+
 protected:
-    
+
     /* mutex lock */
     pthread_mutex_t  m_mutex_lock;
-    
+
 public:
-    
+
     mutex();
 
     void lock();
@@ -55,10 +55,9 @@ private:
 public:
 
     condition_var();
-    
+
     void wait();
     void signal();
-    
+
 };
 
-#endif
