@@ -18,8 +18,7 @@
 //
 //-----------------------------------------------------------------------------
 
-#ifndef SEQ24_OPTIONS
-#define SEQ24_OPTIONS
+#pragma once
 
 #include <gtkmm/button.h>
 #include <gtkmm/window.h>
@@ -34,7 +33,7 @@
 #include <gtkmm/table.h>
 #include <gtkmm/drawingarea.h>
 #include <gtkmm/label.h>
-#include <gtkmm/frame.h> 
+#include <gtkmm/frame.h>
 #include <gtkmm/fileselection.h>
 #include <gtkmm/dialog.h>
 #include <gtkmm/arrow.h>
@@ -54,19 +53,24 @@ using namespace Gtk;
 
 
 
-
 class options : public Gtk::Dialog
 {
 
  private:
 
+#if GTK_MINOR_VERSION < 12
+    Tooltips *m_tooltips;
+#endif
+
     perform *m_perf;
 
     Button  *m_button_ok;
+    Label* interaction_method_label;
+    Label* interaction_method_desc_label;
 
 
     Table   *m_table;
-    
+
     Notebook *m_notebook;
 
     enum button {
@@ -85,17 +89,23 @@ class options : public Gtk::Dialog
     void clock_callback_mod( int a_bus, RadioButton *a_button );
 
     void clock_mod_callback( Adjustment *adj );
-    
-    void input_callback( int a_bus, Button *a_button );
-    void transport_callback( button a_type, Button *a_button );
-    
-public:
-    
-    options( Gtk::Window &parent, perform *a_p );
 
- 
-        
-    
+    void input_callback( int a_bus, Button *a_button );
+
+    void transport_callback( button a_type, Button *a_button );
+
+    void mouse_seq24_callback(Gtk::RadioButton*);
+    void mouse_fruity_callback(Gtk::RadioButton*);
+
+    /*notebook pages*/
+    void add_midi_clock_page();
+    void add_midi_input_page();
+    void add_keyboard_page();
+    void add_mouse_page();
+    void add_jack_sync_page();
+
+public:
+
+    options( Gtk::Window &parent, perform *a_p );
 };
 
-#endif
