@@ -18,27 +18,21 @@
 //
 //-----------------------------------------------------------------------------
 
-#ifndef SEQ24_OPTIONSFILE
-#define SEQ24_OPTIONSFILE
+#pragma once
 
 #include "perform.h"
 #include "configfile.h"
-#include <fstream>
-#include <string>
-#include <list>
 
-class optionsfile  : public configfile
+class optionsfile : public configfile
 {
 
  public:
 
-    optionsfile( string a_name );
-    ~optionsfile( );
-    
+    optionsfile(const Glib::ustring& a_name);
+
     bool parse( perform *a_perf );
     bool write( perform *a_perf );
 
 };
 
 
-#endif 
