@@ -22,13 +22,20 @@
 
 
 seqkeys::seqkeys(sequence *a_seq,
-                 Gtk::Adjustment *a_vadjust ): DrawingArea() 
-{     
-    m_seq = a_seq;
-    
-    m_vadjust = a_vadjust;
-    
-    add_events( Gdk::BUTTON_PRESS_MASK | 
+                 Gtk::Adjustment *a_vadjust ):
+    m_black(Gdk::Color("black")),
+    m_white(Gdk::Color("white")),
+    m_grey(Gdk::Color("grey")),
+    m_seq(a_seq),
+    m_vadjust(a_vadjust),
+    m_scroll_offset_key(0),
+    m_scroll_offset_y(0),
+    m_hint_state(false),
+    m_keying(false),
+    m_scale(0),
+    m_key(0)
+{
+    add_events( Gdk::BUTTON_PRESS_MASK |
 		Gdk::BUTTON_RELEASE_MASK |
 		Gdk::ENTER_NOTIFY_MASK |
 		Gdk::LEAVE_NOTIFY_MASK |
@@ -41,31 +48,18 @@ seqkeys::seqkeys(sequence *a_seq,
     //m_window_x = 10;
     //m_window_y = c_keyarea_y;
 
-    // in the construor you can only allocate colors, 
+    // in the construor you can only allocate colors,
     // get_window() returns 0 because we have not be realized
     Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap();
 
-    m_black = Gdk::Color( "black" );
-    m_white = Gdk::Color( "white" );
-    m_grey = Gdk::Color( "grey" );
-  
     colormap->alloc_color( m_black );
     colormap->alloc_color( m_white );
     colormap->alloc_color( m_grey );
-    
-    m_keying = false;
-    m_hint_state = false;
-
-    m_scroll_offset_key = 0;
-    m_scroll_offset_y = 0;
-
-    m_scale = 0;
-    m_key = 0;
 
     set_double_buffered( false );
 }
 
-void 
+void
 seqkeys::on_realize()
 {
     // we need to do the default realize
@@ -80,7 +74,7 @@ seqkeys::on_realize()
                                    c_keyarea_x,
                                    c_keyarea_y,
                                    -1 );
-  
+
     update_pixmap();
 
     m_vadjust->signal_value_changed().connect( mem_fun( *this, &seqkeys::change_vert ));
@@ -89,7 +83,7 @@ seqkeys::on_realize()
 }
 
 /* sets the music scale */
-void 
+void
 seqkeys::set_scale( int a_scale )
 {
   if ( m_scale != a_scale ){
@@ -100,7 +94,7 @@ seqkeys::set_scale( int a_scale )
 }
 
 /* sets the key */
-void 
+void
 seqkeys::set_key( int a_key )
 {
   if ( m_key != a_key ){
@@ -110,7 +104,7 @@ seqkeys::set_key( int a_key )
 }
 
 
-void 
+void
 seqkeys::reset()
 {
     update_pixmap();
@@ -119,84 +113,84 @@ seqkeys::reset()
 
 
 
-void 
+void
 seqkeys::update_pixmap()
 {
     m_gc->set_foreground(m_black);
     m_pixmap->draw_rectangle(m_gc,true,
                              0,
-                             0, 
-                             c_keyarea_x, 
+                             0,
+                             c_keyarea_x,
                              c_keyarea_y  );
-    
+
     m_gc->set_foreground(m_white);
     m_pixmap->draw_rectangle(m_gc,true,
                              1,
-                             1, 
-                             c_keyoffset_x - 1, 
+                             1,
+                             c_keyoffset_x - 1,
                              c_keyarea_y - 2  );
-    
-    
+
+
     for ( int i=0; i<c_num_keys; i++ )
     {
         m_gc->set_foreground(m_white);
         m_pixmap->draw_rectangle(m_gc,true,
                                  c_keyoffset_x + 1,
-                                 (c_key_y * i) + 1, 
-                                 c_key_x - 2, 
+                                 (c_key_y * i) + 1,
+                                 c_key_x - 2,
                                  c_key_y - 1 );
-        
+
         /* the the key in the octave */
         int key = (c_num_keys - i - 1) % 12;
-        
-        if ( key == 1 || 
-             key == 3 || 
-             key == 6 || 
-             key == 8 || 
+
+        if ( key == 1 ||
+             key == 3 ||
+             key == 6 ||
+             key == 8 ||
              key == 10 ){
-            
+
             m_gc->set_foreground(m_black);
             m_pixmap->draw_rectangle(m_gc,true,
                                      c_keyoffset_x + 1,
-                                     (c_key_y * i) + 2, 
-                                     c_key_x - 3, 
+                                     (c_key_y * i) + 2,
+                                     c_key_x - 3,
                                      c_key_y - 3 );
         }
 
         char notes[20];
-        
+
         if ( key == m_key  ){
-            
-        
-            
+
+
+
             /* notes */
             int octave = ((c_num_keys - i - 1) / 12) - 1;
             if ( octave < 0 )
                 octave *= -1;
-            
-            sprintf( notes, "%2s%1d", c_key_text[key], octave );
-            
+
+            snprintf(notes, sizeof(notes), "%2s%1d", c_key_text[key], octave);
+
             p_font_renderer->render_string_on_drawable(m_gc,
-                                                       2, 
+                                                       2,
                                                        c_key_y * i - 1,
                                                        m_pixmap, notes, font::BLACK );
         }
 
-        //sprintf( notes, "%c %d", c_scales_symbol[m_scale][key], m_scale );
-            
+        //snprintf(notes, sizeof(notes), "%c %d", c_scales_symbol[m_scale][key], m_scale );
+
         //p_font_renderer->render_string_on_drawable(m_gc,
-        //                                             2 + (c_text_x * 4), 
+        //                                             2 + (c_text_x * 4),
         //                                             c_key_y * i - 1,
         //                                             m_pixmap, notes, font::BLACK );
     }
 }
 
-void 
+void
 seqkeys::draw_area()
 {
       update_pixmap();
-      m_window->draw_drawable(m_gc, 
-                              m_pixmap, 
+      m_window->draw_drawable(m_gc,
+                              m_pixmap,
                               0,
                               m_scroll_offset_y,
                               0,
@@ -209,8 +203,8 @@ seqkeys::draw_area()
 bool
 seqkeys::on_expose_event(GdkEventExpose* a_e)
 {
-    m_window->draw_drawable(m_gc, 
-                            m_pixmap, 
+    m_window->draw_drawable(m_gc,
+                            m_pixmap,
                             a_e->area.x,
                             a_e->area.y + m_scroll_offset_y,
                             a_e->area.x,
@@ -222,10 +216,10 @@ seqkeys::on_expose_event(GdkEventExpose* a_e)
 
 
 void
-seqkeys::force_draw( void )
+seqkeys::force_draw()
 {
-    m_window->draw_drawable(m_gc, 
-                            m_pixmap, 
+    m_window->draw_drawable(m_gc,
+                            m_pixmap,
                             0,m_scroll_offset_y,
                             0,0,
                             m_window_x,
@@ -234,10 +228,10 @@ seqkeys::force_draw( void )
 
 
 /* takes screen corrdinates, give us notes and ticks */
-void 
+void
 seqkeys::convert_y( int a_y, int *a_note)
 {
-    *a_note = (c_rollarea_y - a_y - 2) / c_key_y; 
+    *a_note = (c_rollarea_y - a_y - 2) / c_key_y;
 }
 
 
@@ -245,13 +239,13 @@ bool
 seqkeys::on_button_press_event(GdkEventButton *a_e)
 {
     int y,note;
-   
+
     if ( a_e->type == GDK_BUTTON_PRESS ){
 
 	y = (int) a_e->y + m_scroll_offset_y;
 
 	if ( a_e->button == 1 ){
-	    
+	
 	    m_keying = true;
 
 	    convert_y( y,&note );
@@ -266,11 +260,11 @@ seqkeys::on_button_press_event(GdkEventButton *a_e)
 
 bool
 seqkeys::on_button_release_event(GdkEventButton* a_e)
-{   
+{
     if ( a_e->type == GDK_BUTTON_RELEASE ){
 
 	if ( a_e->button == 1 && m_keying ){
-	    
+	
 	    m_keying = false;
 	    m_seq->play_note_off( m_keying_note );
 	}
@@ -284,12 +278,12 @@ seqkeys::on_motion_notify_event(GdkEventMotion* a_p0)
 {
 
     int y, note;
- 
+
     y = (int) a_p0->y + m_scroll_offset_y;
     convert_y( y,&note );
 
     set_hint_key( note );
-    
+
     if ( m_keying ){
 
         if ( note != m_keying_note ){
@@ -330,43 +324,43 @@ seqkeys::on_leave_notify_event(GdkEventCrossing* p0)
 }
 
 /* sets key to grey */
-void 
+void
 seqkeys::set_hint_key( int a_key )
 {
     draw_key( m_hint_key, false );
-    
+
     m_hint_key = a_key;
-    
+
     if ( m_hint_state )
         draw_key( a_key, true );
 }
 
 /* true == on, false == off */
-void 
+void
 seqkeys::set_hint_state( bool a_state )
 {
     m_hint_state = a_state;
-    
+
     if ( !a_state )
         draw_key( m_hint_key, false );
 }
 
 /* a_state, false = normal, true = grayed */
-void 
+void
 seqkeys::draw_key( int a_key, bool a_state )
 {
 
   /* the the key in the octave */
   int key = a_key % 12;
 
-  a_key = c_num_keys - a_key - 1; 
+  a_key = c_num_keys - a_key - 1;
 
-  if ( key == 1 || 
-       key == 3 || 
-       key == 6 || 
-       key == 8 || 
+  if ( key == 1 ||
+       key == 3 ||
+       key == 6 ||
+       key == 8 ||
        key == 10 ){
-    
+
     m_gc->set_foreground(m_black);
   }
   else
@@ -375,18 +369,18 @@ seqkeys::draw_key( int a_key, bool a_state )
 
   m_window->draw_rectangle(m_gc,true,
 			  c_keyoffset_x + 1,
-			  (c_key_y * a_key) + 2 -  m_scroll_offset_y, 
-			  c_key_x - 3, 
+			  (c_key_y * a_key) + 2 -  m_scroll_offset_y,
+			  c_key_x - 3,
 			  c_key_y - 3 );
 
   if ( a_state ){
 
     m_gc->set_foreground(m_grey);
- 
+
     m_window->draw_rectangle(m_gc,true,
 			    c_keyoffset_x + 1,
-			    (c_key_y * a_key) + 2 - m_scroll_offset_y, 
-			    c_key_x - 3, 
+			    (c_key_y * a_key) + 2 - m_scroll_offset_y,
+			    c_key_x - 3,
 			    c_key_y - 3 );
 
   }
@@ -397,12 +391,12 @@ seqkeys::draw_key( int a_key, bool a_state )
 void
 seqkeys::change_vert( )
 {
-   
+
     m_scroll_offset_key = (int) m_vadjust->get_value();
     m_scroll_offset_y = m_scroll_offset_key * c_key_y,
-    
+
     force_draw();
-    
+
 }
 
 
@@ -415,10 +409,10 @@ seqkeys::on_size_allocate(Gtk::Allocation& a_r )
     m_window_x = a_r.get_width();
     m_window_y = a_r.get_height();
 
-  
+
 
     queue_draw();
- 
+
 }
 
 
