@@ -19,8 +19,7 @@
 //-----------------------------------------------------------------------------
 
 
-#ifndef SEQ24_SEQKEYS
-#define SEQ24_SEQKEYS
+#pragma once
 
 #include "sequence.h"
 
@@ -43,27 +42,27 @@
 /* holds the left side piano */
 class seqkeys : public Gtk::DrawingArea
 {
- private: 
+ private:
 
     Glib::RefPtr<Gdk::GC> m_gc;
     Glib::RefPtr<Gdk::Window> m_window;
     Gdk::Color    m_black, m_white, m_grey;
 
     Glib::RefPtr<Gdk::Pixmap> m_pixmap;
-    
+
     sequence *m_seq;
 
     Gtk::Adjustment   *m_vadjust;
-    
+
     int m_scroll_offset_key;
     int m_scroll_offset_y;
 
     int m_window_x;
     int m_window_y;
-    
+
     void on_realize();
     bool on_expose_event(GdkEventExpose* a_ev);
-    bool on_button_press_event(GdkEventButton* a_ev); 
+    bool on_button_press_event(GdkEventButton* a_ev);
     bool on_button_release_event(GdkEventButton* a_ev);
     bool on_motion_notify_event(GdkEventMotion* a_p0);
     bool on_leave_notify_event(GdkEventCrossing* p0);
@@ -85,11 +84,11 @@ class seqkeys : public Gtk::DrawingArea
 
     void draw_key( int a_key, bool a_state );
     void on_size_allocate(Gtk::Allocation&);
-    
-    void change_vert( void );
-    void force_draw( void );
 
-    void update_sizes( void );
+    void change_vert();
+    void force_draw();
+
+    void update_sizes();
 
     void reset();
 
@@ -104,11 +103,10 @@ public:
     seqkeys( sequence *a_seq,
              Gtk::Adjustment *a_vadjust );
 
-    
+
     void set_scale( int a_scale );
     void set_key( int a_key );
 
 
 };
 
-#endif
