@@ -18,23 +18,22 @@
 //
 //-----------------------------------------------------------------------------
 
+
+#pragma once
+
 class sequence;
 
-#ifndef SEQ24_SEQUENCE
-#define SEQ24_SEQUENCE
+#include <string>
+#include <list>
+#include <stack>
 
 #include "event.h"
 #include "midibus.h"
 #include "globals.h"
 #include "mutex.h"
 
-#include <string>
-#include <list>
-#include <stack>
-
 enum draw_type
 {
-
     DRAW_FIN = 0,
     DRAW_NORMAL_LINKED,
     DRAW_NOTE_ON,
@@ -45,27 +44,26 @@ enum draw_type
 class trigger
 {
 public:
-    
+
     long m_tick_start;
     long m_tick_end;
-    
+
     bool m_selected;
-    
+
     long m_offset;
-    
+
     trigger (){
-        
         m_tick_start = 0;
         m_tick_end = 0;
         m_offset = 0;
         m_selected = false;
     };
-    
+
     bool operator< (trigger rhs){
-        
+
         if (m_tick_start < rhs.m_tick_start)
             return true;
-        
+
         return false;
     };
 };
@@ -102,10 +100,13 @@ class sequence
     /* song playback mode mute */
     bool m_song_mute;
 
+    /* polyphonic step edit note counter */
+    int m_notes_on;
+
     /* outputs to sequence to this Bus on midichannel */
     mastermidibus *m_masterbus;
 
-    /* map for noteon, used when muting, to shut off current 
+    /* map for noteon, used when muting, to shut off current
        messages */
     int m_playing_notes[c_midi_notes];
 
@@ -129,7 +130,7 @@ class sequence
     /* anything editing currently ? */
     bool m_editing;
     bool m_raise;
-    
+
     /* named sequence */
     string m_name;
 
@@ -139,7 +140,7 @@ class sequence
 
     long m_trigger_offset;
 
-    /* length of sequence in pulses 
+    /* length of sequence in pulses
        should be powers of two in bars */
     long m_length;
     long m_snap_tick;
@@ -162,9 +163,9 @@ class sequence
     void put_event_on_bus (event * a_e);
 
     /* resetes the location counters */
-    void reset_loop (void);
+    void reset_loop ();
 
-    void remove_all (void);
+    void remove_all ();
 
     /* mutex */
     void lock ();
@@ -175,6 +176,9 @@ class sequence
     void split_trigger( trigger &trig, long a_split_tick);
     void adjust_trigger_offsets_to_legnth( long a_new_len );
     long adjust_offset( long a_offset );
+    void remove( list<event>::iterator i );
+    void remove( event* e );
+
 
   public:
 
@@ -182,12 +186,12 @@ class sequence
      ~sequence ();
 
 
-    void push_undo (void);
-    void pop_undo (void);
-    void pop_redo (void);
+    void push_undo ();
+    void pop_undo ();
+    void pop_redo ();
 
-    void push_trigger_undo (void);
-    void pop_trigger_undo (void);
+    void push_trigger_undo ();
+    void pop_trigger_undo ();
 
     //
     //  Gets and Sets
@@ -198,26 +202,26 @@ class sequence
     void set_name (char *a_name);
 
     void set_measures (long a_length_measures);
-    long get_measures (void);
+    long get_measures ();
 
     void set_bpm (long a_beats_per_measure);
-    long get_bpm (void);
+    long get_bpm ();
 
     void set_bw (long a_beat_width);
-    long get_bw (void);
+    long get_bw ();
     void set_rec_vol (long a_rec_vol);
 
     void set_song_mute (bool a_mute);
-    bool get_song_mute (void);
+    bool get_song_mute ();
 
     /* returns string of name */
-    const char *get_name (void);
+    const char *get_name ();
 
     void set_editing (bool a_edit)
     {
 	m_editing = a_edit;
     };
-    bool get_editing (void)
+    bool get_editing ()
     {
 	return m_editing;
     };
@@ -225,7 +229,7 @@ class sequence
     {
 	m_raise = a_edit;
     };
-    bool get_raise (void)
+    bool get_raise ()
     {
 	return m_raise;
     };
@@ -235,7 +239,7 @@ class sequence
     void set_length (long a_len, bool a_adjust_triggers = true);
     long get_length ();
 
-    /* returns last tick played..  used by 
+    /* returns last tick played..  used by
        editors idle function */
     long get_last_tick ();
 
@@ -246,10 +250,10 @@ class sequence
     bool get_playing ();
     void toggle_playing ();
 
-    void toggle_queued (void);
-    void off_queued (void);
-    bool get_queued (void);
-    long get_queued_tick (void);
+    void toggle_queued ();
+    void off_queued ();
+    bool get_queued ();
+    long get_queued_tick ();
 
     void set_recording (bool);
     bool get_recording ();
@@ -266,7 +270,7 @@ class sequence
     bool is_dirty_edit ();
     bool is_dirty_perf ();
     bool is_dirty_names ();
-    
+
 
     void set_dirty_mp();
     void set_dirty();
@@ -297,29 +301,34 @@ class sequence
     void del_trigger (long a_tick );
     bool get_trigger_state (long a_tick);
     bool select_trigger(long a_tick);
-    bool unselect_triggers (void);
+    bool unselect_triggers ();
+
+    bool intersectTriggers( long position, long& start, long& end );
+    bool intersectNotes( long position, long position_note, long& start, long& end, long& note );
+    bool intersectEvents( long posstart, long posend, long status, long& start );
+
+
+    void del_selected_trigger();
+    void cut_selected_trigger();
+    void copy_selected_trigger();
+    void paste_trigger();
 
-    void del_selected_trigger( void );
-    void cut_selected_trigger( void );
-    void copy_selected_trigger( void );
-    void paste_trigger( void );
-    
     void move_selected_triggers_to(long a_tick, bool a_adjust_offset, int a_which=2);
-    long get_selected_trigger_start_tick( void );
-    long get_selected_trigger_end_tick( void );
+    long get_selected_trigger_start_tick();
+    long get_selected_trigger_end_tick();
 
-    long get_max_trigger (void);
+    long get_max_trigger ();
 
     void move_triggers (long a_start_tick, long a_distance, bool a_direction);
     void copy_triggers (long a_start_tick, long a_distance);
-    void clear_triggers (void);
+    void clear_triggers ();
 
 
-    long get_trigger_offset (void);
+    long get_trigger_offset ();
 
     /* sets the midibus to dump to */
     void set_midi_bus (char a_mb);
-    char get_midi_bus (void);
+    char get_midi_bus ();
 
     void set_master_midi_bus (mastermidibus * a_mmb);
 
@@ -328,9 +337,12 @@ class sequence
         e_select,
         e_select_one,
         e_is_selected,
-        e_would_select
+        e_would_select,
+        e_deselect, // deselect under cursor
+        e_toggle_selection, // sel/deselect under cursor
+        e_remove_one // remove one note under cursor
     };
-    
+
     /* select note events in range, returns number
        selected */
     int select_note_events (long a_tick_s, int a_note_h,
@@ -344,9 +356,9 @@ class sequence
     int get_num_selected_notes ();
     int get_num_selected_events (unsigned char a_status, unsigned char a_cc);
 
-    void select_all (void);
+    void select_all ();
 
-    void copy_selected (void);
+    void copy_selected ();
     void paste_selected (long a_tick, int a_note);
 
     /* returns the 'box' of selected items */
@@ -383,12 +395,12 @@ class sequence
     /* moves note off event */
     void grow_selected (long a_delta_tick);
     void stretch_selected(long a_delta_tick);
-    
+
     /* deletes events */
     void remove_marked();
     void mark_selected();
     void unpaint_all();
-    
+
     /* unselects every event */
     void unselect ();
 
@@ -399,15 +411,15 @@ class sequence
 
     /* resets everything to zero, used when
        sequencer stops */
-    void zero_markers (void);
+    void zero_markers ();
 
-    /* flushes a note to the midibus to preview its 
+    /* flushes a note to the midibus to preview its
        sound, used by the virtual paino */
     void play_note_on (int a_note);
     void play_note_off (int a_note);
 
     /* send a note off for all active notes */
-    void off_playing_notes (void);
+    void off_playing_notes ();
 
     //
     // Drawing functions
@@ -415,11 +427,11 @@ class sequence
 
     /* resets draw marker so calls to getNextnoteEvent
        will start from the first */
-    void reset_draw_marker (void);
-    void reset_draw_trigger_marker (void);
+    void reset_draw_marker ();
+    void reset_draw_trigger_marker ();
 
-    /* each call seqdata( sequence *a_seq, int a_scale );fills the passed refrences with a 
-       events elements, and returns true.  When it 
+    /* each call seqdata( sequence *a_seq, int a_scale );fills the passed refrences with a
+       events elements, and returns true.  When it
        has no more events, returns a false */
     draw_type get_next_note_event (long *a_tick_s,
 				   long *a_tick_f,
@@ -453,4 +465,3 @@ class sequence
     void transpose_notes (int a_steps, int a_scale);
 };
 
-#endif
