@@ -18,14 +18,11 @@
 //
 //-----------------------------------------------------------------------------
 
-#ifndef SEQ24_USERFILE
-#define SEQ24_USERFILE
+#pragma once
 
 #include "perform.h"
 #include "configfile.h"
-#include <fstream>
 #include <string>
-#include <list>
 
 class userfile  : public configfile
 {
@@ -34,11 +31,9 @@ class userfile  : public configfile
 
     userfile( string a_name );
     ~userfile( );
-    
+
     bool parse( perform *a_perf );
     bool write( perform *a_perf );
-
 };
 
 
-#endif 
